@@ -2,29 +2,32 @@
 // used in src/Languages.cpp
 const wxString langNames[] =
 {
-  wxString("Catal", *(new wxCSConv(_T("ISO-8859-1")))),
+  wxString("Catal", wxConvISO8859_1),
   wxT("Česky"),
+  wxT("Dansk"),
   wxT("Deutsch"),
   wxT("English"),
-  wxString("Espaol", *(new wxCSConv(_T("ISO-8859-1")))),
+  wxString("Espaol", wxConvISO8859_1),
   wxT("Euskara"),
-  wxString("Franais", *(new wxCSConv(_T("ISO-8859-1")))),
+  wxString("Franais", wxConvISO8859_1),
   wxT("Italiano"),
-  wxString("日本語 [jp]", *(new wxCSConv(_T("UTF-8")))),
+  wxString("日本語 [jp]", wxConvUTF8),
   wxT("Magyar"),
   wxT("Nederlands"),
   wxT("Polski"),
-  wxString("Portugus", *(new wxCSConv(_T("ISO-8859-1")))),
-  wxString("Portugus (Brasil)", *(new wxCSConv(_T("ISO-8859-1")))),
+  wxString("Portugus", wxConvISO8859_1),
+  wxString("Portugus (Brasil)", wxConvISO8859_1),
   wxString(" [ru]", *(new wxCSConv(_T("KOI8-R")))),
-  wxString("Slovenčina", *(new wxCSConv(_T("UTF-8")))),
-  wxT("Srpski")
+  wxString("Slovenčina", wxConvUTF8),
+  wxT("Srpski"),
+  wxT("Svenska")
 };
 
 const wxLanguage langIds[] =
 {
   wxLANGUAGE_CATALAN,
   wxLANGUAGE_CZECH,
+  wxLANGUAGE_DANISH,
   wxLANGUAGE_GERMAN,
   wxLANGUAGE_ENGLISH,
   wxLANGUAGE_SPANISH,
@@ -32,12 +35,13 @@ const wxLanguage langIds[] =
   wxLANGUAGE_FRENCH,
   wxLANGUAGE_ITALIAN,
   wxLANGUAGE_JAPANESE,
-  wxLANGUAGE_DUTCH,
   wxLANGUAGE_HUNGARIAN,
+  wxLANGUAGE_DUTCH,
   wxLANGUAGE_POLISH,
   wxLANGUAGE_PORTUGUESE,
   wxLANGUAGE_PORTUGUESE_BRAZILIAN,
   wxLANGUAGE_RUSSIAN,
   wxLANGUAGE_SLOVAK,
-  wxLANGUAGE_SERBIAN
+  wxLANGUAGE_SERBIAN,
+  wxLANGUAGE_SWEDISH
 };
