@@ -3,16 +3,17 @@
 // Purpose:     The class to store a DVD Structure (Titles/Menus)
 // Author:      Alex Thuering
 // Created:	29.01.2003
-// RCS-ID:      $Id: DVD.cpp,v 1.33 2007/03/08 20:22:52 ntalex Exp $
+// RCS-ID:      $Id: DVD.cpp,v 1.41 2008/04/18 20:16:01 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
 
 #include "DVD.h"
 #include "Config.h"
-#include <wxXML/xmlhelpr.h>
+#include <wxSVGXML/svgxmlhelpr.h>
 #include <wxVillaLib/utils.h>
 #include <sys/stat.h>
+#include <wxSVG/mediadec_ffmpeg.h>
 
 #define DATA_FILE(fname) wxFindDataFile(_T("data") + wxString(wxFILE_SEP_PATH) + fname)
 
@@ -36,9 +37,9 @@ TextSub::TextSub(wxString filename)
   m_movieHeight = 574;
 }
 
-wxXmlNode* TextSub::GetXML(DVDFileType type)
+wxSvgXmlNode* TextSub::GetXML(DVDFileType type)
 {
-  wxXmlNode* textsubNode = new wxXmlNode(wxXML_ELEMENT_NODE, _T("textsub"));
+  wxSvgXmlNode* textsubNode = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, _T("textsub"));
   textsubNode->AddProperty(_T("filename"), m_filename);
   if (m_characterSet != wxT("ISO8859-1"))
     textsubNode->AddProperty(_T("characterset"), m_characterSet);
@@ -69,7 +70,7 @@ wxXmlNode* TextSub::GetXML(DVDFileType type)
   return textsubNode;
 }
 
-bool TextSub::PutXML(wxXmlNode* node)
+bool TextSub::PutXML(wxSvgXmlNode* node)
 {
   wxString val;
   long lval;
@@ -106,9 +107,9 @@ bool TextSub::PutXML(wxXmlNode* node)
 
 bool TextSub::SaveSpumux(wxString filename)
 {
-  wxXmlDocument xml;
-  wxXmlNode* root = new wxXmlNode(wxXML_ELEMENT_NODE, wxT("subpictures"));
-  wxXmlNode* streamNode = new wxXmlNode(wxXML_ELEMENT_NODE, wxT("stream"));
+  wxSvgXmlDocument xml;
+  wxSvgXmlNode* root = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("subpictures"));
+  wxSvgXmlNode* streamNode = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("stream"));
   streamNode->AddChild(GetXML(DVDAUTHOR_XML));
   root->AddChild(streamNode);
   xml.SetRoot(root);
@@ -126,9 +127,59 @@ Vob::~Vob()
   WX_CLEAR_ARRAY(m_subtitles)
 }
 
-wxXmlNode* Vob::GetXML(DVDFileType type, DVD* dvd)
+long Vob::GetChapterTime(int chapter)
 {
-  wxXmlNode* node = new wxXmlNode(wxXML_ELEMENT_NODE, wxT("vob"));
+  long time = 0;
+  if (chapter > 0)
+  {
+    wxString timeStr = m_chapters;
+    while (timeStr.Find(wxT(',')) != wxNOT_FOUND && --chapter)
+      timeStr = timeStr.AfterFirst(wxT(','));
+    timeStr = timeStr.BeforeFirst(wxT(','));
+    while (1)
+    {
+      time *= 60;
+      if (timeStr.Find(wxT(':')) == wxNOT_FOUND)
+      {
+        double t = 0;
+        if (timeStr.ToDouble(&t))
+          time += (int) (t*1000);
+        break;
+      }
+      else
+      {
+        long t = 0;
+        if (timeStr.BeforeFirst(wxT(':')).ToLong(&t))
+          time += t;
+        timeStr = timeStr.AfterFirst(wxT(':'));
+      }
+    }
+  }
+  return time;
+}
+
+wxString Vob::GetVideoFrameUri(int chapter, long position)
+{
+  wxString uri = GetFilename().length() ? GetFilename() : GetVideoFilename();
+  if (position == -1 && chapter > 0)
+    position = GetChapterTime(chapter);
+  if (position >= 0 && uri.length() > 0)
+    uri += wxT('#') + wxString::Format(wxT("%d"), position);
+  return uri;
+}
+
+void Vob::UpdateInfo()
+{
+  wxFfmpegMediaDecoder ffmpeg;
+  bool ok = m_filename.length() && ffmpeg.Load(m_filename)
+      || m_videoFilename.length() && ffmpeg.Load(m_videoFilename);
+  m_duration = ok ? ffmpeg.GetDuration() : 0;
+  m_videoSize = ok ? ffmpeg.GetVideoSize() : wxSize();
+}
+
+wxSvgXmlNode* Vob::GetXML(DVDFileType type, DVD* dvd)
+{
+  wxSvgXmlNode* node = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("vob"));
   wxString fname = GetFilename();
   if (type == DVDAUTHOR_XML && GetTmpFilename().length())
     fname = GetTmpFilename();
@@ -140,8 +191,8 @@ wxXmlNode* Vob::GetXML(DVDFileType type, DVD* dvd)
 	  XmlWriteValue(node, wxT("video"), GetVideoFilename());
     for (unsigned int i=0; i<GetAudioFilenames().Count(); i++)
     {
-      wxXmlNode* audioNode = new wxXmlNode(wxXML_ELEMENT_NODE, wxT("audio"));
-      audioNode->AddChild(new wxXmlNode(wxXML_TEXT_NODE, wxEmptyString,
+      wxSvgXmlNode* audioNode = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("audio"));
+      audioNode->AddChild(new wxSvgXmlNode(wxSVGXML_TEXT_NODE, wxEmptyString,
         GetAudioFilenames()[i]));
       node->AddChild(audioNode);
     }
@@ -171,9 +222,9 @@ wxXmlNode* Vob::GetXML(DVDFileType type, DVD* dvd)
     {
       wxString chapters;
       int t = 1;
-      for (unsigned i=1; i<GetSlideshow()->Count(); i++)
+      for (unsigned i=1; i<GetSlideshow()->Count()/5; i++)
       {
-        t += GetSlideshow()->GetDuration();
+        t += GetSlideshow()->GetDuration()*5;
         int h = t/3600;
         int m = (t%3600)/60;
         int s = t%60;
@@ -188,13 +239,15 @@ wxXmlNode* Vob::GetXML(DVDFileType type, DVD* dvd)
   return node;
 }
 
-bool Vob::PutXML(wxXmlNode* node)
+bool Vob::PutXML(wxSvgXmlNode* node)
 {
   wxString val;
   long lval;
   
   node->GetPropVal(wxT("file"), &m_filename);
-  wxXmlNode* child = node->GetChildren();
+  if (m_filename.length())
+     UpdateInfo();
+  wxSvgXmlNode* child = node->GetChildren();
   while (child)
   {
 	if (child->GetName() == wxT("video"))
@@ -296,11 +349,14 @@ Slideshow* Pgc::GetSlideshow()
   return NULL;
 }
 
-wxXmlNode* Pgc::GetXML(DVDFileType type, DVD* dvd, wxString nextTitle)
+wxSvgXmlNode* Pgc::GetXML(DVDFileType type, DVD* dvd, wxString nextTitle)
 {
-  wxXmlNode* node = new wxXmlNode(wxXML_ELEMENT_NODE, wxT("pgc"));
+  wxSvgXmlNode* node = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("pgc"));
   if (m_entry.length())
 	node->AddProperty(wxT("entry"), m_entry);
+
+  if (m_palette.length())
+	node->AddProperty(wxT("palette"), m_palette);
   
   if (GetMenu() && type == DVDAUTHOR_XML)
 	GetMenu()->GetXML(type, dvd->GetPlayAllRegister(), node);
@@ -325,14 +381,15 @@ wxXmlNode* Pgc::GetXML(DVDFileType type, DVD* dvd, wxString nextTitle)
   return node;
 }
 
-bool Pgc::PutXML(wxXmlNode* node)
+bool Pgc::PutXML(wxSvgXmlNode* node)
 {
   WX_CLEAR_ARRAY(GetVobs());
   
   wxString val;
   node->GetPropVal(wxT("entry"), &m_entry);
+  node->GetPropVal(wxT("palette"), &m_palette);
   
-  wxXmlNode* child = node->GetChildren();
+  wxSvgXmlNode* child = node->GetChildren();
   Menu* menu = NULL; // old
   
   while (child)
@@ -409,9 +466,9 @@ wxArrayString Audio::GetSamplerateStrings(wxString def)
   return ret;
 }
 
-wxXmlNode* Audio::GetXML(DVDFileType type)
+wxSvgXmlNode* Audio::GetXML(DVDFileType type)
 {
-  wxXmlNode* node = new wxXmlNode(wxXML_ELEMENT_NODE, wxT("audio"));
+  wxSvgXmlNode* node = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("audio"));
   
   if (m_format.length())
 	node->AddProperty(wxT("format"), m_format);
@@ -434,7 +491,7 @@ wxXmlNode* Audio::GetXML(DVDFileType type)
   return node;
 }
 
-bool Audio::PutXML(wxXmlNode* node)
+bool Audio::PutXML(wxSvgXmlNode* node)
 {
   if (node == NULL || node->GetName() !=  wxT("audio"))
 	return false;
@@ -479,9 +536,9 @@ wxArrayString Video::GetWidescreenStrings(wxString def)
   return ret;
 }
 
-wxXmlNode* Video::GetXML(DVDFileType type)
+wxSvgXmlNode* Video::GetXML(DVDFileType type)
 {
-  wxXmlNode* node = new wxXmlNode(wxXML_ELEMENT_NODE, wxT("video"));
+  wxSvgXmlNode* node = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("video"));
   
   if (m_format.length())
 	node->AddProperty(wxT("format"), m_format);
@@ -502,7 +559,7 @@ wxXmlNode* Video::GetXML(DVDFileType type)
   return node;
 }
 
-bool Video::PutXML(wxXmlNode* node)
+bool Video::PutXML(wxSvgXmlNode* node)
 {
   if (node == NULL || node->GetName() !=  wxT("video"))
 	return false;
@@ -519,9 +576,9 @@ bool Video::PutXML(wxXmlNode* node)
  
 ////////////////////////////////// Menus //////////////////////////////////////
 
-wxXmlNode* Menus::GetXML(DVDFileType type, DVD* dvd)
+wxSvgXmlNode* Menus::GetXML(DVDFileType type, DVD* dvd)
 {
-  wxXmlNode* node = new wxXmlNode(wxXML_ELEMENT_NODE, wxT("menus"));
+  wxSvgXmlNode* node = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("menus"));
   if (GetAudio().GetXML(type))
 	node->AddChild(GetAudio().GetXML(type));
   if (GetVideo().GetXML(type))
@@ -531,12 +588,12 @@ wxXmlNode* Menus::GetXML(DVDFileType type, DVD* dvd)
   return node;
 }
 
-bool Menus::PutXML(wxXmlNode* node)
+bool Menus::PutXML(wxSvgXmlNode* node)
 {
   if (node == NULL || node->GetName() !=  wxT("menus"))
 	return false;
   WX_CLEAR_ARRAY(*this);
-  wxXmlNode* child = node->GetChildren();
+  wxSvgXmlNode* child = node->GetChildren();
   while (child)
   {
 	if (child->GetName() == wxT("audio"))
@@ -562,9 +619,9 @@ bool Menus::PutXML(wxXmlNode* node)
 
 ////////////////////////////////// Titles ////////////////////////////////////
 
-wxXmlNode* Titles::GetXML(DVDFileType type, DVD* dvd, int nextTitleset)
+wxSvgXmlNode* Titles::GetXML(DVDFileType type, DVD* dvd, int nextTitleset)
 {
-  wxXmlNode* node = new wxXmlNode(wxXML_ELEMENT_NODE, wxT("titles"));
+  wxSvgXmlNode* node = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("titles"));
   if (GetAudio().GetXML(type))
 	node->AddChild(GetAudio().GetXML(type));
   if (GetVideo().GetXML(type))
@@ -580,11 +637,11 @@ wxXmlNode* Titles::GetXML(DVDFileType type, DVD* dvd, int nextTitleset)
   return node;
 }
 
-bool Titles::PutXML(wxXmlNode* node)
+bool Titles::PutXML(wxSvgXmlNode* node)
 {
   if (node == NULL || node->GetName() !=  wxT("titles"))
 	return false;
-  wxXmlNode* child = node->GetChildren();
+  wxSvgXmlNode* child = node->GetChildren();
   while (child)
   {
 	if (child->GetName() == wxT("audio"))
@@ -610,9 +667,9 @@ bool Titles::PutXML(wxXmlNode* node)
 
 ////////////////////////////////// Titleset //////////////////////////////////
 
-wxXmlNode* Titleset::GetXML(DVDFileType type, DVD* dvd, int nextTitleset)
+wxSvgXmlNode* Titleset::GetXML(DVDFileType type, DVD* dvd, int nextTitleset)
 {
-  wxXmlNode* node = new wxXmlNode(wxXML_ELEMENT_NODE, wxT("titleset"));
+  wxSvgXmlNode* node = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("titleset"));
   if (m_menus.Count() || type != DVDAUTHOR_XML)
 	node->AddChild(m_menus.GetXML(type, dvd));
   else if (dvd->IsEmptyMenuEnabled() &&	dvd->GetVmgm().Count())
@@ -629,11 +686,11 @@ wxXmlNode* Titleset::GetXML(DVDFileType type, DVD* dvd, int nextTitleset)
   return node;
 }
 
-bool Titleset::PutXML(wxXmlNode* node)
+bool Titleset::PutXML(wxSvgXmlNode* node)
 {
   if (node == NULL || node->GetName() !=  wxT("titleset"))
 	return false;
-  wxXmlNode* child = node->GetChildren();
+  wxSvgXmlNode* child = node->GetChildren();
   while (child)
   {
 	if (child->GetName() == wxT("menus"))
@@ -657,6 +714,46 @@ bool Titleset::PutXML(wxXmlNode* node)
   return true;
 }
 
+////////////////////////////////// VMGM //////////////////////////////////////
+
+wxSvgXmlNode* Vmgm::GetXML(DVDFileType type, DVD* dvd)
+{
+  wxSvgXmlNode* node = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("vmgm"));
+  // add fgc
+  if (GetFpc().length())
+  	XmlWriteValue(node, wxT("fpc"), GetFpc());
+  
+  // add menus
+  if (type == DVDSTYLER_XML || Count())
+	node->AddChild(Menus::GetXML(type, dvd));
+  else if (dvd->IsEmptyMenuEnabled()
+      && dvd->GetTitlesets().Count() && dvd->GetTitlesets()[0]->GetMenus().Count())
+  {
+	// create empty vmgm with post command
+	Menus menus;
+	Pgc* pgc = new Pgc;
+	pgc->SetPreCommands(wxT("jump titleset 1 menu;"));
+    pgc->GetVobs().Add(Vob::CreateEmptyVob(dvd->GetVideoFormat(), dvd->GetAudioFormat()));
+	menus.Add(pgc);
+	node->AddChild(menus.GetXML(DVDAUTHOR_XML, dvd));
+  }
+  return node;
+}
+
+bool Vmgm::PutXML(wxSvgXmlNode* node)
+{	
+  wxSvgXmlNode* child = node->GetChildren();
+  while (child)
+  {
+	if (child->GetName() == wxT("menus"))
+	  Menus::PutXML(child);
+	else if (child->GetName() == wxT("fpc"))
+	  SetFpc(child->GetChildren()->GetContent());
+	child = child->GetNext();
+  }
+  return true;
+}
+
 ////////////////////////////////// DVD //////////////////////////////////////
 
 DVD::DVD()
@@ -680,18 +777,20 @@ int DVD::AddTitleset()
   return GetTitlesets().Count();
 }
 
-int DVD::AddMenu(VideoFormat format, int tsi)
-{
-  Menu* menu = new Menu((VideoFormat)format);
-  Vob* vob = new Vob(menu);
-  vob->SetPause(-1);
-  Pgc* pgc = new Pgc;
-  pgc->GetVobs().Add(vob);
-  PgcArray& pgcs = GetPgcArray(tsi, true);
-  if (pgcs.GetCount() == 0) // set aspect ratio
-  	pgcs.GetVideo().SetAspect(wxT("4:3"));
-  pgcs.Add(pgc);
-  return pgcs.Count()-1;
+int DVD::AddMenu(VideoFormat format, int tsi) {
+	AddMenu(new Menu((VideoFormat)format), tsi);
+}
+
+int DVD::AddMenu(Menu* menu, int tsi) {
+	Vob* vob = new Vob(menu);
+	vob->SetPause(-1);
+	Pgc* pgc = new Pgc;
+	pgc->GetVobs().Add(vob);
+	PgcArray& pgcs = GetPgcArray(tsi, true);
+	if (pgcs.GetCount() == 0) // set aspect ratio
+		pgcs.GetVideo().SetAspect(wxT("4:3"));
+	pgcs.Add(pgc);
+	return pgcs.Count()-1;
 }
 
 int DVD::AddPgc(int tsi, bool menu, Pgc* pgc)
@@ -750,14 +849,14 @@ Menu* DVD::GetMenu(int tsi, int pgci)
 bool DVD::Open(wxString fname)
 {
   WX_CLEAR_ARRAY(GetTitlesets());
-  wxXmlDocument xml;
+  wxSvgXmlDocument xml;
   if (!xml.Load(fname))
   {
 	wxLogError(_("Can't load xml-file"));
 	return false;
   }
   
-  wxXmlNode* root = xml.GetRoot();
+  wxSvgXmlNode* root = xml.GetRoot();
   if (root == NULL || root->GetName() !=  wxT("dvdstyler"))
   {
 	wxLogError(_("This is not a DVDStyler project file"));
@@ -768,21 +867,25 @@ bool DVD::Open(wxString fname)
   
   wxString val;
   long lval;
+  
+  int format = 0;
+  if (root->GetPropVal(wxT("format"), &val) && val.ToLong(&lval))
+  	format = int(lval);
   if (root->GetPropVal(wxT("jumppad"), &val) && val.ToLong(&lval))
 	m_jumppad = int(lval);
   if (root->GetPropVal(wxT("emptyMenu"), &val) && val.ToLong(&lval))
 	m_emptyMenu = int(lval);
   if (root->GetPropVal(wxT("videoFormat"), &val) && val.ToLong(&lval))
-	m_videoFormat = VideoFormat(lval);
+	m_videoFormat = VideoFormat(lval + (format == 2 ? 1 : 0));
   if (root->GetPropVal(wxT("audioFormat"), &val) && val.ToLong(&lval))
-	m_audioFormat = AudioFormat(lval);
+	m_audioFormat = AudioFormat(lval + (format == 2 ? 1 : 0));
   
-  wxXmlNode* child = root->GetChildren();
+  wxSvgXmlNode* child = root->GetChildren();
   while (child)
   {
 	if (child->GetName() ==  wxT("vmgm"))
 	{
-	  if (!GetVmgm().PutXML(child->GetChildren()))
+	  if (!GetVmgm().PutXML(child))
 	  {
 		wxLogError(_("Can't load vmgm menus"));
 		return false;
@@ -808,17 +911,15 @@ bool DVD::Open(wxString fname)
 bool DVD::Save(wxString fname)
 {
   m_playAllRegister = -1;
-  wxXmlDocument xml;
-  wxXmlNode* root = new wxXmlNode(wxXML_ELEMENT_NODE, wxT("dvdstyler"));
-  root->AddProperty(wxT("format"), wxT("2"));
+  wxSvgXmlDocument xml;
+  wxSvgXmlNode* root = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("dvdstyler"));
+  root->AddProperty(wxT("format"), wxT("3"));
   root->AddProperty(wxT("name"), GetName());
   root->AddProperty(wxT("jumppad"), m_jumppad ? wxT("1") : wxT("0"));
   root->AddProperty(wxT("emptyMenu"), m_emptyMenu ? wxT("1") : wxT("0"));
   root->AddProperty(wxT("videoFormat"), wxString::Format(wxT("%d"), m_videoFormat));
   root->AddProperty(wxT("audioFormat"), wxString::Format(wxT("%d"), m_audioFormat));
-  wxXmlNode* vmgmNode = new wxXmlNode(wxXML_ELEMENT_NODE, wxT("vmgm"));
-  vmgmNode->AddChild(GetVmgm().GetXML(DVDSTYLER_XML, this));
-  root->AddChild(vmgmNode);
+  root->AddChild(GetVmgm().GetXML(DVDSTYLER_XML, this));
   for (int i=0; i<(int)m_titlesets.GetCount(); i++)
 	if (m_titlesets[i]->GetMenus().Count() || m_titlesets[i]->GetTitles().Count())
 	  root->AddChild(m_titlesets[i]->GetXML(DVDSTYLER_XML, this));
@@ -832,24 +933,10 @@ bool DVD::SaveDVDAuthor(wxString fname)
   if (HasPlayAllButton())
   	m_playAllRegister = FindFreeRegister();
   // save config for dvdauthor
-  wxXmlDocument xml;
-  wxXmlNode* root = new wxXmlNode(wxXML_ELEMENT_NODE, wxT("dvdauthor"));
+  wxSvgXmlDocument xml;
+  wxSvgXmlNode* root = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, wxT("dvdauthor"));
   root->AddProperty(wxT("jumppad"), m_jumppad ? wxT("1") : wxT("0"));
-  wxXmlNode* vmgmNode = new wxXmlNode(wxXML_ELEMENT_NODE, wxT("vmgm"));
-  if (GetVmgm().Count())
-	vmgmNode->AddChild(GetVmgm().GetXML(DVDAUTHOR_XML, this));
-  else if (IsEmptyMenuEnabled() &&
-	GetTitlesets().Count() && GetTitlesets()[0]->GetMenus().Count())
-  {
-	// create empty vmgm with post command
-	Menus menus;
-	Pgc* pgc = new Pgc;
-	pgc->SetPreCommands(wxT("jump titleset 1 menu;"));
-    pgc->GetVobs().Add(Vob::CreateEmptyVob(GetVideoFormat(), GetAudioFormat()));
-	menus.Add(pgc);
-	vmgmNode->AddChild(menus.GetXML(DVDAUTHOR_XML, this));
-  }
-  root->AddChild(vmgmNode);
+  root->AddChild(GetVmgm().GetXML(DVDAUTHOR_XML, this));
   for (int i=0; i<(int)m_titlesets.GetCount(); i++) {
   	int nextTitleset = i<(int)m_titlesets.GetCount()-1 && !m_titlesets[i+1]->IsEmpty() ? i+2 : -1;
 	if (!m_titlesets[i]->IsEmpty())
