@@ -3,13 +3,14 @@
 // Purpose:     Main window
 // Author:      Alex Thuering
 // Created:     10.10.2003
-// RCS-ID:      $Id: MainWin.cpp,v 1.78 2006/12/06 14:53:37 ntalex Exp $
+// RCS-ID:      $Id: MainWin.cpp,v 1.88 2008/04/28 20:54:12 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
 
 #include "MainWin.h"
 #include "BurnDlg.h"
+#include "DonateDlg.h"
 #include "ProgressDlg.h"
 #include "About.h"
 #include "NewProjectDlg.h"
@@ -26,21 +27,19 @@
 
 #ifdef __WXMSW__
 #include <wx/msw/registry.h>
-#undef  wxBITMAP
-#define wxBITMAP(icon) wxBitmap(icon##_xpm)
 #else
 #define wxDirDialogDefaultFolderStr _T("~")
 #endif
 
-#include "rc/new.xpm"
-#include "rc/open.xpm"
-#include "rc/save.xpm"
-#include "rc/run.xpm"
+#include "rc/new.png.h"
+#include "rc/open.png.h"
+#include "rc/save.png.h"
+#include "rc/run.png.h"
 #ifndef __WXMSW__
-#include "rc/folder1.xpm"
-#include "rc/folder2.xpm"
+#include "rc/folder1.png.h"
+#include "rc/folder2.png.h"
 #endif
-#include "rc/files.xpm"
+#include "rc/files.png.h"
 
 #define BACKGROUNDS_DIR wxFindDataDirectory(_T("backgrounds"))
 #define BUTTONS_DIR wxFindDataDirectory(_T("buttons"))
@@ -104,8 +103,7 @@ BEGIN_EVENT_TABLE(MainWin, wxFrame)
 END_EVENT_TABLE()
 
 MainWin::MainWin(): wxFrame(NULL, -1, _T(""), wxDefaultPosition, wxDefaultSize,
-  wxDEFAULT_FRAME_STYLE), m_dvd(NULL)
-{
+		wxDEFAULT_FRAME_STYLE), m_dvd(NULL) {
     // begin wxGlade: MainWin::MainWin
     m_splitterTitle = new wxSplitterWindow(this, SPLIITTER_TITLE_ID, wxDefaultPosition, wxDefaultSize, wxCLIP_CHILDREN);
     m_splitterDir = new wxSplitterWindow(m_splitterTitle, -1, wxDefaultPosition, wxDefaultSize, wxCLIP_CHILDREN);
@@ -119,7 +117,7 @@ MainWin::MainWin(): wxFrame(NULL, -1, _T(""), wxDefaultPosition, wxDefaultSize,
     wxglade_tmp_menu_1->Append(wxID_SAVE, _("&Save"), wxT(""), wxITEM_NORMAL);
     wxglade_tmp_menu_1->Append(wxID_SAVEAS, _("S&ave as..."), wxT(""), wxITEM_NORMAL);
     wxglade_tmp_menu_1->AppendSeparator();
-    wxglade_tmp_menu_1->Append(MENU_BURN_ID, _("&Burn dvd..."), wxT(""), wxITEM_NORMAL);
+    wxglade_tmp_menu_1->Append(MENU_BURN_ID, _("&Burn DVD..."), wxT(""), wxITEM_NORMAL);
     wxglade_tmp_menu_1->AppendSeparator();
     wxglade_tmp_menu_1->Append(wxID_EXIT, _("&Exit"), wxT(""), wxITEM_NORMAL);
     m_menubar->Append(wxglade_tmp_menu_1, _("&File"));
@@ -150,6 +148,8 @@ MainWin::MainWin(): wxFrame(NULL, -1, _T(""), wxDefaultPosition, wxDefaultSize,
     do_layout();
     // end wxGlade
     
+    m_menuEditor->SetTitlesetManager(m_titlesetManager);
+    
     // set fiter
     m_thumbnails->AddCustomExtention(wxT("sub"));
     m_thumbnails->AddCustomExtention(wxT("srt"));
@@ -165,11 +165,12 @@ MainWin::MainWin(): wxFrame(NULL, -1, _T(""), wxDefaultPosition, wxDefaultSize,
 	
 	// set file browser dir
 	wxString fbDir = s_config.GetFileBrowserDir();
-	if (!fbDir.length() || fbDir.Strip(wxString::both) == _T("$HOME"))
-	  fbDir = wxGetHomeDir();
-#if wxCHECK_VERSION(2,5,0)
+	if (fbDir.length() == 0)
+		fbDir = s_config.GetFileBrowserLastDir();
+	if (fbDir.length() == 0)
+		fbDir = wxGetHomeDir();
 	m_dirTree->SetPath(fbDir);
-#endif
+	m_lastDir = s_config.GetLastProjectDir();
 	
 	// show Backgrouns
 	m_dirBar->ToggleTool(BACKGROUNS_BT_ID, true);
@@ -196,7 +197,7 @@ void MainWin::set_properties()
     // begin wxGlade: MainWin::set_properties
     SetTitle(_("DVDStyler"));
     SetSize(wxSize(1000, 740));
-    int m_statusbar_widths[] = { -1, 200 };
+    int m_statusbar_widths[] = { -1, 250 };
     m_statusbar->SetStatusWidths(2, m_statusbar_widths);
     const wxString m_statusbar_fields[] = {
         wxT(""),
@@ -217,20 +218,20 @@ void MainWin::set_properties()
 #endif
 	
 	// tool bar
-	m_toolbar->AddTool(wxID_NEW,  wxT(""), wxBITMAP(new),  wxNullBitmap,
+	m_toolbar->AddTool(wxID_NEW,  wxT(""), wxBITMAP_FROM_MEMORY(new),  wxNullBitmap,
 	  wxITEM_NORMAL, _("New"), wxT(""));
-	m_toolbar->AddTool(wxID_OPEN, wxT(""), wxBITMAP(open), wxNullBitmap,
+	m_toolbar->AddTool(wxID_OPEN, wxT(""), wxBITMAP_FROM_MEMORY(open), wxNullBitmap,
 	  wxITEM_NORMAL, _("Open..."), wxT(""));
-	m_toolbar->AddTool(wxID_SAVE, wxT(""), wxBITMAP(save), wxNullBitmap,
+	m_toolbar->AddTool(wxID_SAVE, wxT(""), wxBITMAP_FROM_MEMORY(save), wxNullBitmap,
 	  wxITEM_NORMAL, _("Save"), wxT(""));
-	m_toolbar->AddTool(MENU_BURN_ID,  wxT(""), wxBITMAP(run),  wxNullBitmap,
+	m_toolbar->AddTool(MENU_BURN_ID,  wxT(""), wxBITMAP_FROM_MEMORY(run),  wxNullBitmap,
 	  wxITEM_NORMAL, _("Burn..."), wxT(""));
 	m_toolbar->Realize();
 	
 	// dir tree
 #ifndef __WXMSW__
-	m_dirTree->GetTreeCtrl()->GetImageList()->Replace(0, SetDefaultMask(wxBITMAP(folder1)));
-    m_dirTree->GetTreeCtrl()->GetImageList()->Replace(1, SetDefaultMask(wxBITMAP(folder2)));
+	m_dirTree->GetTreeCtrl()->GetImageList()->Replace(0, SetDefaultMask(wxBITMAP_FROM_MEMORY(folder1)));
+    m_dirTree->GetTreeCtrl()->GetImageList()->Replace(1, SetDefaultMask(wxBITMAP_FROM_MEMORY(folder2)));
 #endif
 	m_dirTree->SetWindowStyle(m_dirTree->GetWindowStyle() | wxCLIP_CHILDREN);
 	
@@ -241,7 +242,7 @@ void MainWin::set_properties()
     // dir bar
     m_dirBar->SetMargins(4, 4);
     wxVerticalToolbar* vertToolBar = new wxVerticalToolbar(m_dirBar);
-    vertToolBar->AddTool(DIR_TREE_BT_ID, _("Directories"), wxBITMAP(files),
+    vertToolBar->AddTool(DIR_TREE_BT_ID, _("Directories"), wxBITMAP_FROM_MEMORY(files),
 	  wxITEM_RADIO);
     vertToolBar->AddTool(BACKGROUNS_BT_ID, _("Backgrounds"), wxNullBitmap,
 	  wxITEM_RADIO);
@@ -387,6 +388,7 @@ void MainWin::OnDirSelected(wxTreeEvent& event)
     return;
   m_thumbnails->ShowFileNames();
   m_thumbnails->ShowDir(m_dirTree->GetPath());
+  s_config.SetFileBrowserLastDir(m_dirTree->GetPath());
 }
 
 void MainWin::OnThumbInfo(wxCommandEvent& event)
@@ -395,9 +397,11 @@ void MainWin::OnThumbInfo(wxCommandEvent& event)
   if (m_thumbnails->GetPointed() != -1)
   {
 	if (m_dirBar->GetToolState(BACKGROUNS_BT_ID))
-	  text = _("Double click to set the background.");
-	else if (!m_dirBar->GetToolState(BUTTONS_BT_ID))
-	  text = m_thumbnails->GetPointedItem()->GetFilename();
+		text = _("Double click to set the background.");
+	else if (m_dirBar->GetToolState(BUTTONS_BT_ID))
+		text = m_thumbnails->GetPointedItem()->GetFilename().AfterLast(wxFILE_SEP_PATH);
+	else
+		text = m_thumbnails->GetPointedItem()->GetFilename();
   }
   m_statusbar->SetStatusText(text, 0);
 }
@@ -528,6 +532,7 @@ void MainWin::Open(wxString filename)
     m_dvd = dvd;
     m_filename = filename;
     m_lastDir = wxPathOnly(m_filename);
+    s_config.SetLastProjectDir(m_lastDir);
 	UpdateDVD();
 	SetChanged(false);
   }
@@ -543,6 +548,8 @@ void MainWin::UpdateDVD()
   m_titlesetManager->SetDVD(m_dvd);
   m_titlesetManager->UpdateItems();
   m_titlesetManager->SelectFirstMenu();
+  wxCommandEvent evt;
+  OnChangedDVD(evt);
 }
 
 
@@ -587,7 +594,7 @@ void MainWin::OnSelectMenu(wxCommandEvent& event)
 void MainWin::OnChangedDVD(wxCommandEvent& event)
 {
   SetChanged(true);
-  m_statusbar->SetStatusText(wxString::Format(_("Total size: %dMB"),
+  m_statusbar->SetStatusText(wxString::Format(_("Estimated total size: %dMB"),
     m_dvd->GetSize()/1024), 1);
 }
 
@@ -599,7 +606,7 @@ bool MainWin::TestSaved()
 	if (!fname.length())
 	  fname = _("Untitled");
 	int save = wxMessageBox(
-	  wxString::Format(_("The file '%s' is modifed, save changes?"), fname.c_str()),
+	  wxString::Format(_("The file '%s' was modified, save changes?"), fname.c_str()),
 	  _("Confirm"), wxYES_NO|wxCANCEL|wxICON_QUESTION, this);
 	if (save == wxYES)
 	{
@@ -627,8 +634,6 @@ void MainWin::OnOpen(wxCommandEvent& event)
 {
   if (!TestSaved())
 	return;
-  if (!m_lastDir.length())
-	m_lastDir = wxGetHomeDir();
   wxFileDialog dlg( this, _("Open a DVDStyler project file"), m_lastDir,
 	_T(""), _T("*.xml"), wxOPEN);
   dlg.Centre();
@@ -652,8 +657,6 @@ void MainWin::OnSaveAs(wxCommandEvent& event)
   if (m_dvd->GetTitlesets().Count() ||
       m_dvd->GetVmgm().Count())
   {
-	if (!m_lastDir.length())
-	m_lastDir = wxGetHomeDir();
 	wxFileDialog dlg(this, _("Save a DVDStyler project file"), m_lastDir,
 	  _T("dvd.xml"), _T("*.xml"), wxSAVE | wxOVERWRITE_PROMPT);
 	dlg.Centre();
@@ -661,6 +664,7 @@ void MainWin::OnSaveAs(wxCommandEvent& event)
 	{
 	  m_filename = dlg.GetPath();
 	  m_lastDir = wxPathOnly(m_filename);
+	  s_config.SetLastProjectDir(m_lastDir);
 	  if (m_dvd->Save(m_filename))
 		SetChanged(false);
 	}
@@ -684,15 +688,24 @@ void MainWin::OnBurn(wxCommandEvent& event)
 	  }
 	}
   }
-  
-  if (vobcnt == 0)
-  {
+  // check if user added some titles
+  if (vobcnt == 0) {
 	wxMessageBox(
 	  _("There are no titles specified. To add a title please drag\n a video file into the bottom section of the window."),
 	  _("Burn"), wxOK|wxICON_ERROR, this);
 	return;
   }
-		
+  // check if the last title doesn't contain only video
+  PgcArray& pgcs = m_dvd->GetTitlesets().Last()->GetTitles();
+  Pgc* pgc = pgcs.Count()>0 ? pgcs.Last() : NULL;
+  Vob* vob = pgc && pgc->GetVobs().Count()>0 ? pgc->GetVobs().Last() : NULL;
+  if (vob && vob->GetVideoFilename().length() && !vob->GetAudioFilenames().Count()) {
+	  wxMessageBox(wxString(_("The last title isn't complete.\n")) +
+			  _("Please add an audio track to it."),
+			  _("Burn"), wxOK|wxICON_ERROR, this);
+	  return;
+  }
+  
   BurnDlg burnDlg(this);
   if (burnDlg.ShowModal() != wxID_OK)
 	return;
@@ -700,6 +713,9 @@ void MainWin::OnBurn(wxCommandEvent& event)
   // Progress dialog
   ProgressDlg progressDlg(this);
   progressDlg.Start(&burnDlg, m_dvd);
+  
+  DonateDlg donateDlg(this);
+  donateDlg.ShowModal();
 }
 
 void MainWin::OnExit(wxCommandEvent& event)
