@@ -3,7 +3,7 @@
 // Purpose:     The class to store a DVD Menu
 // Author:      Alex Thuering
 // Created:	04.11.2003
-// RCS-ID:      $Id: Menu.cpp,v 1.59 2007/03/08 19:52:03 ntalex Exp $
+// RCS-ID:      $Id: Menu.cpp,v 1.64 2008/03/31 17:54:54 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -14,10 +14,14 @@
 #include <wxVillaLib/SConv.h>
 #include <wxVillaLib/utils.h>
 #include <wxSVG/svg.h>
-#include <wxXML/xmlhelpr.h>
+#include <wxSVG/SVGCanvas.h>
+#include <wxSVGXML/svgxmlhelpr.h>
 #include <wx/mstream.h>
+#include <wx/filename.h>
 
 #define OBJECTS_DIR wxFindDataDirectory(_T("objects"))
+#define BACKGROUNDS_DIR wxFindDataDirectory(_T("backgrounds"))
+
 #define BUTTON_MAX_COLORS 4
 
 Menu::Menu(VideoFormat videoFormat)
@@ -80,12 +84,19 @@ wxSize Menu::GetResolution()
   return wxSize(720,576); // vfPAL
 }
 
-void Menu::SetBackground(wxString fileName)
+void Menu::SetBackground(wxString fileName) {
+	SetBackground(m_svg->GetRootElement(), fileName);
+}
+wxString Menu::GetBackground() {
+	return GetBackground(m_svg->GetRootElement());
+}
+
+void Menu::SetBackground(wxSVGSVGElement* root, wxString fileName)
 {
   bool image = wxImage::FindHandler(fileName.AfterLast(wxT('.')).Lower(), -1);
   
   // test if element exists
-  wxSVGElement* bgElement = m_svg->GetElementById(wxT("background"));
+  wxSVGElement* bgElement = (wxSVGElement*) root->GetElementById(wxT("background"));
   if (!bgElement ||
       (image && bgElement->GetDtd() != wxSVG_IMAGE_ELEMENT) ||
       (!image && bgElement->GetDtd() != wxSVG_VIDEO_ELEMENT))
@@ -105,8 +116,7 @@ void Menu::SetBackground(wxString fileName)
       ((wxSVGVideoElement*) bgElement)->SetHeight(GetResolution().GetHeight());
     }
     bgElement->SetId(wxT("background"));
-    m_svg->GetRootElement()->InsertChild(bgElement,
-      m_svg->GetRootElement()->GetChildren());
+    root->InsertChild(bgElement, root->GetChildren());
   }
   
   // set href
@@ -116,10 +126,9 @@ void Menu::SetBackground(wxString fileName)
     ((wxSVGVideoElement*) bgElement)->SetHref(fileName);
 }
 
-wxString Menu::GetBackground()
+wxString Menu::GetBackground(wxSVGSVGElement* root)
 {
-  wxSVGElement* bgElement =
-    (wxSVGElement*) m_svg->GetElementById(wxT("background"));
+  wxSVGElement* bgElement = (wxSVGElement*) root->GetElementById(wxT("background"));
   if (bgElement)
   {
     if (bgElement->GetDtd() == wxSVG_IMAGE_ELEMENT)
@@ -198,7 +207,7 @@ void Menu::RemoveChangeable(wxSVGElement* element, MenuObject* obj)
   {
     wxSVGElement* elem = child;
     child = (wxSVGElement*) child->GetNext();
-    if (elem->GetType() != wxXML_ELEMENT_NODE)
+    if (elem->GetType() != wxSVGXML_ELEMENT_NODE)
       continue;
     // don't remove def elements
     if (IsDefElement(elem))
@@ -237,7 +246,7 @@ bool Menu::RemoveNotChangeable(wxSVGElement* element, MenuObject* obj)
   {
     wxSVGElement* elem = child;
     child = (wxSVGElement*) child->GetNext();
-    if (elem->GetType() != wxXML_ELEMENT_NODE)
+    if (elem->GetType() != wxSVGXML_ELEMENT_NODE)
     {
       elem->GetParent()->RemoveChild(elem);
       continue;
@@ -272,38 +281,51 @@ bool Menu::RemoveNotChangeable(wxSVGElement* element, MenuObject* obj)
   return element->GetChildren() == NULL; // return if has no children
 }
 
-wxSVGSVGElement* Menu::GetSVGCopy() {
-	wxSVGSVGElement* svgNode = (wxSVGSVGElement*) m_svg->GetRoot()->CloneNode();
-	// remove selection rectangle
-	wxXmlElement* elem = svgNode->GetElementById(wxT("selection"));
-	if (elem)
-		elem->GetParent()->RemoveChild(elem);
-	// remove safeTV rectangle
-	elem = svgNode->GetElementById(wxT("safeTV"));
-	if (elem)
-		elem->GetParent()->RemoveChild(elem);
-	// remove grid
-	elem = svgNode->GetElementById(wxT("grid"));
-	if (elem)
-		elem->GetParent()->RemoveChild(elem);
-	return svgNode;
+void CopyCachedImages(wxSVGDocument* doc, wxSVGElement* parent1, wxSVGElement* parent2)
+{
+  wxSVGElement* elem1 = (wxSVGElement*) parent1->GetChildren();
+  wxSVGElement* elem2 = (wxSVGElement*) parent2->GetChildren();
+  while (elem1 && elem2)
+  {
+    if (elem1->GetType() == wxSVGXML_ELEMENT_NODE
+        && elem1->GetDtd() == wxSVG_IMAGE_ELEMENT)
+    {
+      wxSVGImageElement* img1 = (wxSVGImageElement*) elem1;
+      wxSVGImageElement* img2 = (wxSVGImageElement*) elem2;
+      if (img1->GetHref().GetAnimVal().length() && img1->GetCanvasItem() != NULL)
+          img2->SetCanvasItem(doc->GetCanvas()->CreateItem(img1));
+    } else if (elem1->GetChildren())
+      CopyCachedImages(doc, elem1, elem2);
+    elem1 = (wxSVGElement*) elem1->GetNext();
+    elem2 = (wxSVGElement*) elem2->GetNext();
+  }
 }
 
-////////////////////////////// Render ////////////////////////////////////////
-
-wxImage Menu::RenderImage(MenuDrawType drawType, int width, int height)
-{
-  wxSVGDocument svg;
-  svg.AppendChild(GetSVGCopy());
+wxSVGSVGElement* Menu::GetSVGCopy(MenuDrawType drawType) {
+  wxSVGSVGElement* svgNode = (wxSVGSVGElement*) m_svg->GetRoot()->CloneNode();
+  CopyCachedImages(m_svg, (wxSVGElement*) m_svg->GetRoot(), svgNode);
+  // remove selection rectangle
+  wxSvgXmlElement* elem = svgNode->GetElementById(wxT("selection"));
+  if (elem)
+    elem->GetParent()->RemoveChild(elem);
+  // remove safeTV rectangle
+  elem = svgNode->GetElementById(wxT("safeTV"));
+  if (elem)
+    elem->GetParent()->RemoveChild(elem);
+  // remove grid
+  elem = svgNode->GetElementById(wxT("grid"));
+  if (elem)
+    elem->GetParent()->RemoveChild(elem);
   
   if (drawType == mdBACKGROUND)
   {
+    // remove changeable elements of objects
     for (int i=0; i<GetObjectsCount(); i++)
     {
       MenuObject* obj = GetObject(i);
       if (obj->IsButton())
       {
-        wxSVGElement* elem = svg.GetElementById(wxT("s_") + obj->GetId());
+        wxSVGElement* elem = (wxSVGElement*) svgNode->GetElementById(wxT("s_") + obj->GetId());
         if (elem)
           RemoveChangeable(elem, obj);
       }
@@ -313,7 +335,8 @@ wxImage Menu::RenderImage(MenuDrawType drawType, int width, int height)
            drawType == mdBUTTONS_HIGHLIGHTED ||
            drawType == mdBUTTONS_SELECTED)
   {
-    wxSVGElement* bgElement = svg.GetElementById(wxT("background"));
+    // remove background and static elements of objects
+    wxSVGElement* bgElement = (wxSVGElement*) svgNode->GetElementById(wxT("background"));
     wxSVGRectElement* bgRect = (wxSVGRectElement*) bgElement;
     if (!bgElement || bgElement->GetDtd() != wxSVG_RECT_ELEMENT)
     {
@@ -323,8 +346,7 @@ wxImage Menu::RenderImage(MenuDrawType drawType, int width, int height)
       bgRect->SetId(wxT("background"));
       bgRect->SetWidth(GetResolution().GetWidth());
       bgRect->SetHeight(GetResolution().GetHeight());
-      svg.GetRootElement()->InsertChild(bgRect,
-        svg.GetRootElement()->GetChildren());
+      svgNode->InsertChild(bgRect, svgNode->GetChildren());
     }
     bgRect->SetFill(wxSVGPaint(m_transpColour.Red(), m_transpColour.Green(), m_transpColour.Blue()));
     for (int i=0; i<GetObjectsCount(); i++)
@@ -332,8 +354,7 @@ wxImage Menu::RenderImage(MenuDrawType drawType, int width, int height)
       MenuObject* obj = GetObject(i);
       if (obj->IsButton())
       {
-        wxSVGSVGElement* symbol = (wxSVGSVGElement*)
-          svg.GetElementById(wxT("s_") + obj->GetId());
+        wxSVGSVGElement* symbol = (wxSVGSVGElement*) svgNode->GetElementById(wxT("s_") + obj->GetId());
         if (symbol)
           RemoveNotChangeable(symbol, obj);
         for (int i=0; i<obj->GetObjectParamsCount(); i++)
@@ -354,16 +375,25 @@ wxImage Menu::RenderImage(MenuDrawType drawType, int width, int height)
       }
       else
       {
-        wxSVGElement* elem = svg.GetElementById(obj->GetId());
+        wxSVGElement* elem = (wxSVGElement*) svgNode->GetElementById(obj->GetId());
         if (elem && elem->GetParent())
           elem->GetParent()->RemoveChild(elem);
-        elem = svg.GetElementById(wxT("s_") + obj->GetId());
+        elem = (wxSVGElement*) svgNode->GetElementById(wxT("s_") + obj->GetId());
         if (elem && elem->GetParent())
           elem->GetParent()->RemoveChild(elem);
       }
     }
   }
   
+  return svgNode;
+}
+
+////////////////////////////// Render ////////////////////////////////////////
+
+wxImage Menu::RenderImage(MenuDrawType drawType, int width, int height)
+{
+  wxSVGDocument svg;
+  svg.AppendChild(GetSVGCopy(drawType));
   return svg.Render(width, height);
 }
 
@@ -372,6 +402,13 @@ wxImage Menu::GetImage(int width, int height)
   return RenderImage(mdALL, width, height);
 }
 
+wxSVGDocument* Menu::GetBackgroundSVG()
+{
+  wxSVGDocument* svg = new wxSVGDocument;
+  svg->AppendChild(GetSVGCopy(mdBACKGROUND));
+  return svg;
+}
+
 bool Menu::ReduceColours()
 {
   // create 3d palette
@@ -543,10 +580,10 @@ wxString Menu::AddObject(wxString fileName, wxString param, int x, int y)
 bool Menu::SaveSpumux(wxString fileName,
   wxString btFile, wxString hlFile, wxString selFile)
 {
-  wxXmlDocument xml;
-  wxXmlNode* root = new wxXmlNode(wxXML_ELEMENT_NODE, _T("subpictures"));
-  wxXmlNode* streamNode = new wxXmlNode(wxXML_ELEMENT_NODE, _T("stream"));
-  wxXmlNode* spuNode = new wxXmlNode(wxXML_ELEMENT_NODE, _T("spu"));
+  wxSvgXmlDocument xml;
+  wxSvgXmlNode* root = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, _T("subpictures"));
+  wxSvgXmlNode* streamNode = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, _T("stream"));
+  wxSvgXmlNode* spuNode = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, _T("spu"));
   if (GetStartTime().Length())
 	spuNode->AddProperty(_T("start"), GetStartTime());
   if (GetEndTime().Length())
@@ -556,125 +593,141 @@ bool Menu::SaveSpumux(wxString fileName,
   spuNode->AddProperty(_T("select"), selFile);
   spuNode->AddProperty(_T("transparent"), SConv::ToString(m_transpColour, false));
   spuNode->AddProperty(_T("force"), _T("yes"));
+  // buttons
   for (int i=0; i<(int)GetObjectsCount(); i++)
   {
     MenuObject* obj = GetObject(i);
 	if (obj->IsButton())
 	  spuNode->AddChild(obj->GetXML(SPUMUX_XML));
   }
+  // actions
+  for (unsigned int i=0; i<m_actions.Count(); i++)
+    spuNode->AddChild(m_actions[i]->GetXML(SPUMUX_XML));
   streamNode->AddChild(spuNode);
   root->AddChild(streamNode);
   xml.SetRoot(root);
   return xml.Save(fileName);
 }
 
-wxXmlNode* Menu::GetXML(DVDFileType type, int playAllRegister, wxXmlNode* node)
-{
-  if (node == NULL)
-	node = new wxXmlNode(wxXML_ELEMENT_NODE, _T("menu"));
+wxSvgXmlNode* Menu::GetXML(DVDFileType type, int playAllRegister, wxSvgXmlNode* node) {
+	if (node == NULL)
+		node = new wxSvgXmlNode(wxSVGXML_ELEMENT_NODE, _T("menu"));
   
-  if (type == DVDSTYLER_XML)
-  {
-	node->AddProperty(_T("videoFormat"), (m_videoFormat == vfPAL) ? _T("PAL") : _T("NTSC"));
-	if (GetBackground().length())
-	  node->AddProperty(_T("bgFile"), GetBackground());
-	else
-	  node->AddProperty(_T("bgColour"), SConv::ToString(GetBackgroundColour()));
-    
-    // add svg
-    if (m_svg && m_svg->GetRoot())
-      node->AppendChild(GetSVGCopy());
-  }
-  
-  // add buttons info (action, etc.)
-  for (int i=0; i<GetObjectsCount(); i++)
-  {
-	MenuObject* obj = GetObject(i);
-	if (type == DVDSTYLER_XML || obj->IsButton())
-	  node->AddChild(obj->GetXML(type, false, playAllRegister));
-  }
-  
-  return node;
-}
-
-bool Menu::PutXML(wxXmlNode* node)
-{
-  if (node->GetName() == _T("spumux"))
-    node = node->GetChildren();
-  if (node != NULL && node->GetName() == _T("menu"))
-  {
-    wxString val;
-    m_videoFormat = vfPAL;
-    if (node->GetPropVal(_T("videoFormat"), &val) && val == _T("NTSC"))
-      m_videoFormat = vfNTSC;
-      
-    wxXmlNode* svgNode = XmlFindNode(node, wxT("svg"));
-    if (svgNode)
-    {
-      wxXmlDocument xml;
-      xml.SetRoot(svgNode->CloneNode());
-      wxMemoryOutputStream output;
-      xml.Save(output);
-#if wxCHECK_VERSION(2,6,0)
-      wxMemoryInputStream input(output);
-      m_svg->Load(input);
-#else
-      char* data = new char[output.GetSize()];
-      output.CopyTo(data, output.GetSize());
-      wxMemoryInputStream input(data, output.GetSize());
-      m_svg->Load(input);
-      delete[] data;
-#endif
-    }
-    else // deprecated
-    {
-      if (node->GetPropVal(_T("bgFile"), &val))
-        SetBackground(val);
-      else if (node->GetPropVal(_T("bgColour"), &val))
-        SetBackgroundColour(SConv::ToColour(val));
-    }
-	
-	for (wxXmlNode* child = node->GetChildren(); child != NULL; child = child->GetNext())
-	  if (child->GetType() == wxXML_ELEMENT_NODE)
-		AddObject(child);
-	// fix for old version (<=1.5b5)
-	if (m_svg->GetElementById(wxT("objects")) == NULL) {
-	  	wxSVGGElement* objectsElem = new wxSVGGElement;
-	  	objectsElem->SetId(wxT("objects"));
-  		m_svg->GetRoot()->AppendChild(objectsElem);
-  		wxSVGGElement* buttonsElem = new wxSVGGElement;
-  		buttonsElem->SetId(wxT("buttons"));
-  		m_svg->GetRoot()->AppendChild(buttonsElem);
-  		m_svg->GetRootElement()->GetChildren();
-  		for (unsigned int i = 0; i < m_objects.GetCount(); i++) {
-  			MenuObject& obj = *m_objects.Item(i);
-  			wxSVGElement* useElem = m_svg->GetElementById(obj.GetId());
-  			useElem->GetParent()->RemoveChild(useElem);
-	  		if (obj.IsButton())
-	  			buttonsElem->AppendChild(useElem);
-	  		else
-	  			objectsElem->AppendChild(useElem);
-  		}
+	if (type == DVDSTYLER_XML) {
+		node->AddProperty(_T("videoFormat"), (m_videoFormat == vfPAL) ? _T("PAL") : _T("NTSC"));
+		if (GetStartTime().Length() && GetStartTime() != wxT("00:00:00.00"))
+			node->AddProperty(_T("start"), GetStartTime());
+		if (GetEndTime().Length())
+			node->AddProperty(_T("end"), GetEndTime());
+		
+		// add svg
+		if (m_svg && m_svg->GetRoot()) {
+			wxSVGSVGElement* svg = GetSVGCopy();
+			// fix path
+			wxString bgFile = GetBackground(svg);
+			if (bgFile.length() > 0) {
+				wxString dir = BACKGROUNDS_DIR;
+				if (bgFile.StartsWith(dir))
+	 				SetBackground(svg, bgFile.substr(dir.length()));
+			}
+			node->AppendChild(svg);
+		}
 	}
-  }
-  return true;
+	
+	// add buttons info (action, etc.)
+	for (int i=0; i<GetObjectsCount(); i++) {
+		MenuObject* obj = GetObject(i);
+		if (type == DVDSTYLER_XML || obj->IsButton())
+			node->AddChild(obj->GetXML(type, false, playAllRegister));
+	}
+	
+	// actions
+	for (unsigned int i=0; i<m_actions.Count(); i++)
+		node->AddChild(m_actions[i]->GetXML(type));
+	
+	return node;
 }
 
-wxString Menu::AddObject(wxXmlNode* node, bool fixPosition)
+bool Menu::PutXML(wxSvgXmlNode* node) {
+	if (node->GetName() == _T("spumux"))
+		node = node->GetChildren();
+	if (node != NULL && node->GetName() == _T("menu")) {
+		wxString val;
+		m_videoFormat = vfPAL;
+		if (node->GetPropVal(_T("videoFormat"), &val) && val == _T("NTSC"))
+			m_videoFormat = vfNTSC;
+		node->GetPropVal(_T("startTime"), &m_startTime);
+		node->GetPropVal(_T("endTime"), &m_endTime);
+		
+		wxSvgXmlNode* svgNode = XmlFindNode(node, wxT("svg"));
+		if (svgNode) {
+			wxSvgXmlDocument xml;
+			xml.SetRoot(svgNode->CloneNode());
+			wxMemoryOutputStream output;
+			xml.Save(output);
+			wxMemoryInputStream input(output);
+			m_svg->Load(input);
+			// fix path
+			if (GetBackground().length() > 0) {
+				wxFileName fn(GetBackground());
+				if (fn.IsRelative())
+					SetBackground(BACKGROUNDS_DIR + GetBackground());
+			}
+	    } else { // deprecated
+	    	if (node->GetPropVal(_T("bgFile"), &val))
+	    		SetBackground(val);
+	    	else if (node->GetPropVal(_T("bgColour"), &val))
+	    		SetBackgroundColour(SConv::ToColour(val));
+		}
+		
+		for (wxSvgXmlNode* child = node->GetChildren(); child != NULL; child = child->GetNext())
+			if (child->GetType() == wxSVGXML_ELEMENT_NODE)
+				AddObject(child);
+		// fix for old version (<=1.5b5)
+		if (m_svg->GetElementById(wxT("objects")) == NULL) {
+		  	wxSVGGElement* objectsElem = new wxSVGGElement;
+		  	objectsElem->SetId(wxT("objects"));
+	  		m_svg->GetRoot()->AppendChild(objectsElem);
+	  		wxSVGGElement* buttonsElem = new wxSVGGElement;
+	  		buttonsElem->SetId(wxT("buttons"));
+	  		m_svg->GetRoot()->AppendChild(buttonsElem);
+	  		m_svg->GetRootElement()->GetChildren();
+	  		for (unsigned int i = 0; i < m_objects.GetCount(); i++) {
+	  			MenuObject& obj = *m_objects.Item(i);
+	  			wxSVGElement* useElem = m_svg->GetElementById(obj.GetId());
+	  			useElem->GetParent()->RemoveChild(useElem);
+	  			if (obj.IsButton())
+		  			buttonsElem->AppendChild(useElem);
+		  		else
+		  			objectsElem->AppendChild(useElem);
+	  		}
+		}
+	}
+	return true;
+}
+
+wxString Menu::AddObject(wxSvgXmlNode* node, bool fixPosition)
 {
-  if (node->GetName() != _T("button") && node->GetName() != _T("object"))
+    if (node->GetName() == _T("action")) {
+        MenuAction* action = new MenuAction();
+        if (!action->PutXML(node)) {
+            delete action;
+            return wxT("");
+        }
+        m_actions.Add(action);
+        return action->GetId();
+    } else if (node->GetName() == _T("button") || node->GetName() == _T("object")) {
+        MenuObject* obj = new MenuObject(this);
+        if (!obj->PutXML(node)) {
+            delete obj;
+            return wxT("");
+        }
+        if (fixPosition) // for copy & paste
+            FixPosition(obj);
+        m_objects.Add(obj);
+        return obj->GetId();
+    }
     return wxT("");
-  
-  MenuObject* obj = new MenuObject(this);
-  if (!obj->PutXML(node))
-  {
-    delete obj;
-	return wxT("");
-  }
-  if (fixPosition) // for copy & paste
-	FixPosition(obj);
-  m_objects.Add(obj);
-  return obj->GetId();
 }
 
 void Menu::FixPosition(MenuObject* obj)
