@@ -3,7 +3,7 @@
 // Purpose:     DVD menu button properties dialog
 // Author:      Alex Thuering
 // Created:	20.11.2003
-// RCS-ID:      $Id: MenuObjectPropDlg.cpp,v 1.9 2007/01/12 15:51:35 ntalex Exp $
+// RCS-ID:      $Id: MenuObjectPropDlg.cpp,v 1.16 2008/03/31 17:55:25 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -13,32 +13,38 @@
 #include <wx/tokenzr.h>
 #include <stdint.h>
 
-enum
-{
-  JUMP_ACTION_RADIO_ID = 7900,
-  CUSTOM_ACTION_RADIO_ID,
-  TSI_CHOICE_ID,
-  PGCI_CHOICE_ID,
-  AUTO_SIZE_RADIO_ID,
-  CUSTOM_SIZE_RADIO_ID
+enum {
+	JUMP_ACTION_RADIO_ID = 7900,
+	CUSTOM_ACTION_RADIO_ID,
+	TSI_CHOICE_ID,
+	PGCI_CHOICE_ID,
+	AUTO_SIZE_RADIO_ID,
+	CUSTOM_SIZE_RADIO_ID,
+	IMAGE_RADIO_ID,
+	VIDEOFRAME_RADIO_ID,
+	VIDEOFRAME_CHOICE_ID
 };
 
 BEGIN_EVENT_TABLE(MenuObjectPropDlg, wxPropDlg)
-  EVT_RADIOBUTTON(JUMP_ACTION_RADIO_ID, MenuObjectPropDlg::OnJumpActionSelected)
-  EVT_RADIOBUTTON(CUSTOM_ACTION_RADIO_ID, MenuObjectPropDlg::OnCustomActionSelected)
-  EVT_RADIOBUTTON(AUTO_SIZE_RADIO_ID, MenuObjectPropDlg::OnDefaultSizeSelected)
-  EVT_RADIOBUTTON(CUSTOM_SIZE_RADIO_ID, MenuObjectPropDlg::OnCustomSizeSelected)
-  EVT_CHOICE(TSI_CHOICE_ID, MenuObjectPropDlg::OnChangeTitleset)
-  EVT_CHOICE(PGCI_CHOICE_ID, MenuObjectPropDlg::OnChangeTarget)
+	EVT_RADIOBUTTON(JUMP_ACTION_RADIO_ID, MenuObjectPropDlg::OnJumpActionSelected)
+	EVT_RADIOBUTTON(CUSTOM_ACTION_RADIO_ID, MenuObjectPropDlg::OnCustomActionSelected)
+	EVT_RADIOBUTTON(AUTO_SIZE_RADIO_ID, MenuObjectPropDlg::OnDefaultSizeSelected)
+	EVT_RADIOBUTTON(CUSTOM_SIZE_RADIO_ID, MenuObjectPropDlg::OnCustomSizeSelected)
+	EVT_CHOICE(TSI_CHOICE_ID, MenuObjectPropDlg::OnChangeTitleset)
+	EVT_CHOICE(PGCI_CHOICE_ID, MenuObjectPropDlg::OnChangeTarget)
+	EVT_CHOICE(VIDEOFRAME_CHOICE_ID, MenuObjectPropDlg::OnSelectVideoFrame)
 END_EVENT_TABLE()
 
 MenuObjectPropDlg::MenuObjectPropDlg(wxWindow* parent,
-  wxString id, Menu* menu, DVD* dvd, int tsi, int pgci):
-    wxPropDlg(parent), m_id(id), m_menu(menu), m_dvd(dvd),
-    m_tsi(tsi), m_pgci(pgci)
-{
-  m_object = menu->GetObject(id);
-  Create();
+		wxString id, Menu* menu, DVD* dvd, int tsi, int pgci): wxPropDlg(parent) {
+	m_id = id;
+	m_menu = menu;
+	m_dvd = dvd;
+	m_tsi = tsi;
+	m_pgci = pgci;
+	m_videoFrameRadio = NULL;
+	m_object = menu->GetObject(id);
+	Create();
 }
 
 void MenuObjectPropDlg::CreatePropPanel(wxSizer* sizer)
@@ -55,10 +61,10 @@ void MenuObjectPropDlg::CreatePropPanel(wxSizer* sizer)
     AddRadioProp(grid, _("Jump to"), !m_object->HasCustomAction(), wxRB_GROUP,
       false, JUMP_ACTION_RADIO_ID);
     wxBoxSizer* jumpSizer = new wxBoxSizer(wxHORIZONTAL);
-    AddChoiceProp(jumpSizer, wxT(""), wxT(""), wxArrayString(), TSI_CHOICE_ID);
+    AddChoiceProp(jumpSizer, wxT(""), wxT(""), wxArrayString(), -1, true, TSI_CHOICE_ID);
     m_titlesetChoice = (wxChoice*) m_controls[m_controls.Count()-1];
     UpdateTitlesets();
-    AddChoiceProp(jumpSizer, wxT(""), wxT(""), wxArrayString(), PGCI_CHOICE_ID);
+    AddChoiceProp(jumpSizer, wxT(""), wxT(""), wxArrayString(), -1, true, PGCI_CHOICE_ID);
     m_targetChoice = (wxChoice*) m_controls[m_controls.Count()-1];
     UpdateTargets();
     AddChoiceProp(jumpSizer, wxT(""), wxT(""), wxArrayString());
@@ -126,9 +132,11 @@ void MenuObjectPropDlg::CreatePropPanel(wxSizer* sizer)
       lastChangeable = param->changeable;
     }
     
+    wxString title = wxGetTranslation((const wxChar*)param->title.GetData()) + wxString(wxT(":"));
+    
     if (param->changeable && param->type == _T("colour"))
     {
-      AddColourProp(grid, param->title + wxT(":"), m_object->GetParamColour(param->name));
+      AddColourProp(grid, title, m_object->GetParamColour(param->name));
       AddColourProp(grid, wxT(""), m_object->GetParamColour(param->name, mbsHIGHLIGHTED));
       AddColourProp(grid, wxT(""), m_object->GetParamColour(param->name, mbsSELECTED));
     }
@@ -136,51 +144,94 @@ void MenuObjectPropDlg::CreatePropPanel(wxSizer* sizer)
     {
       if (param->type == _T("text"))
       {
-        AddText(grid, param->title + wxT(":"));
+        AddText(grid, title);
         wxBoxSizer* sizer2 = new wxBoxSizer(wxHORIZONTAL);
         AddTextProp(sizer2, wxT(""), m_object->GetParam(param->name));
         AddFontProp(sizer2, wxT(""), m_object->GetParamFont(param->name), _("Font..."));
         grid->Add(sizer2, 0, wxEXPAND);
-      }
-      else if (param->type == _T("string"))
-        AddTextProp(grid, param->title + wxT(":"), m_object->GetParam(param->name));
-      else if (param->type.Mid(0,7) == _T("integer"))
-      {
+      } else if (param->type == _T("string")) {
+        AddTextProp(grid, title, m_object->GetParam(param->name));
+      } else if (param->type == _T("decimal")) {
+        AddTextProp(grid, title, m_object->GetParam(param->name), false, 100);
+      } else if (param->type.Mid(0,7) == _T("integer")) {
         long min = 0, max = 999;
         if (param->type.Find(wxT('(')) != -1 && param->type.Find(wxT(')')) != -1)
         {
           param->type.AfterFirst(wxT('(')).BeforeFirst(wxT(',')).Strip(wxString::both).ToLong(&min);
           param->type.BeforeFirst(wxT(')')).AfterFirst(wxT(',')).Strip(wxString::both).ToLong(&max);
         }
-        AddSpinProp(grid, param->title + wxT(":"), m_object->GetParamInt(param->name), min, max);
+        AddSpinProp(grid, title, m_object->GetParamInt(param->name), min, max,
+        		max < 1000 ? 54 : 80);
+      } else if (param->type == _T("image")) {
+    	  wxString imageFile = m_object->GetParam(param->name);
+    	  if (m_object->IsButton()) {
+	          bool hasVideo = GetVideoFilename().length();
+	          wxString fname = imageFile.Find(wxT('#')) != -1 ? imageFile.BeforeLast(wxT('#')) : imageFile;
+	          bool image = !hasVideo || fname != GetVideoFilename();
+	          AddRadioProp(grid, wxString(_("Image")) + wxT(":"), image, wxRB_GROUP, false, IMAGE_RADIO_ID);
+	          AddFileProp(grid, wxT(""), image ? imageFile : wxT(""), wxFD_OPEN, wxT("..."),
+	    		_("Image Files ") + wxImage::GetImageExtWildcard()
+	     		+ wxT("|") + wxString(_("All Files")) + wxT(" (*.*)|*.*"));
+	          AddRadioProp(grid, wxString(_("Video Frame")) + wxT(":"), !image, 0, false, VIDEOFRAME_RADIO_ID);
+	          m_videoFrameRadio = (wxRadioButton*) m_controls[m_controls.Count()-1];
+	          if (!hasVideo)
+	            m_videoFrameRadio->Enable(false);
+	          bool videoFrameAuto = imageFile == GetVideoFilename(true);
+	          const wxString frameChoices[] = { _("Auto"), _("Custom")};
+	          wxSizer* frameSizer = AddChoiceProp(grid, wxT(""), frameChoices[videoFrameAuto ? 0 : 1],
+	        		  wxArrayString(2, frameChoices), 100, false, VIDEOFRAME_CHOICE_ID);
+	          GetLastControl()->SetName(wxString::Format(wxT("%d"), GetLastControlIndex()));
+	          frameSizer->AddSpacer(4);
+	          long videoFrame = 0;
+	          if (!image) {
+	        	  wxString vf = imageFile.Find(wxT('#')) != -1 ? imageFile.AfterLast(wxT('#')) : wxT("");
+	        	  if (vf.length() > 0)
+	        		  vf.ToLong(&videoFrame);
+	          }
+	          wxSizer* frameSizer2 = AddSpinProp(frameSizer, wxT(""), videoFrame, 0, 9999999, 100, false);
+	          GetLastControl()->SetName(wxString::Format(wxT("videoFrame_%d"), GetLastControlIndex()-1));
+	          GetLastControl()->Enable(!videoFrameAuto);
+	          frameSizer2->AddSpacer(4);
+	          AddText(frameSizer2, wxT("ms"));
+	          frameSizer2->Add(0, 0, 1, wxEXPAND);
+	      } else
+	    	  AddFileProp(grid, title, imageFile, wxFD_OPEN,
+	        		wxT("..."), _("Image Files ") + wxImage::GetImageExtWildcard()
+	        		+ wxT("|") + wxString(_("All Files")) + wxT(" (*.*)|*.*"));
+      }
+      else if (param->type == _T("colour")) {
+    	  int opacity = 100;
+    	  wxString opacityStr = m_object->GetParam(param->name, wxT("-opacity"));
+    	  if (opacityStr.length() > 0) {
+    		  double dval;
+    		  opacityStr.ToDouble(&dval);
+    		  opacity = (int) (dval*100);
+    	  }
+    	  AddColourProp(grid, title, m_object->GetParamColour(param->name), opacity);
       }
-      else if (param->type == _T("image"))
-        AddFileProp(grid, param->title + wxT(":"), m_object->GetParam(param->name));
-      else if (param->type == _T("colour"))
-        AddColourProp(grid, param->title + wxT(":"), m_object->GetParamColour(param->name));
     }
   }
   
   // object location
   AddStaticLine(mainSizer, _("Location and size"));
-  m_posCtrlNum = m_controls.Count();
   grid = new wxFlexGridSizer(1, 4, 4, 16);
   AddSpinProp(grid, _("x:"), m_object->GetX(), 0, m_menu->GetResolution().GetWidth()-8);
   AddSpinProp(grid, _("y:"), m_object->GetY(), 0, m_menu->GetResolution().GetHeight()-8);
   mainSizer->Add(grid, 0, wxEXPAND|wxBOTTOM, 8);
   
   // object size
-  grid = new wxFlexGridSizer(2, 5, 4, 16);
-  grid->AddGrowableCol(1);
+  grid = new wxFlexGridSizer(2, 6, 4, 16);
+  grid->AddGrowableCol(5);
   AddRadioProp(grid, _("Default width and height"), m_object->IsDefaultSize(),
     wxRB_GROUP, false, AUTO_SIZE_RADIO_ID);
-  grid->Add(10,10); grid->Add(10,10); grid->Add(10,10); grid->Add(10,10);
+  grid->Add(10,10); grid->Add(10,10); grid->Add(10,10); grid->Add(10,10); grid->Add(10,10);
   AddRadioProp(grid, _("Custom"), !m_object->IsDefaultSize(), 0,
     false, CUSTOM_SIZE_RADIO_ID);
   AddSpinProp(grid, _("width:"), m_object->GetWidth(), 0, m_menu->GetResolution().GetWidth());
   m_widthEdit = (wxTextCtrl*) m_controls[m_controls.Count()-1];
   AddSpinProp(grid, _("height:"), m_object->GetHeight(), 0, m_menu->GetResolution().GetHeight());
   m_heightEdit = (wxTextCtrl*) m_controls[m_controls.Count()-1];
+  grid->Add(10,10);
   mainSizer->Add(grid, 0, wxEXPAND|wxBOTTOM, 8);
   wxCommandEvent evt;
   if (m_object->IsDefaultSize())
@@ -260,6 +311,8 @@ void MenuObjectPropDlg::UpdateTargets()
     }
   }
   ctrl->SetSelection(sel);
+  if (m_videoFrameRadio)
+    m_videoFrameRadio->Enable(GetVideoFilename().length());
 }
 
 void MenuObjectPropDlg::UpdateChapters()
@@ -299,6 +352,8 @@ void MenuObjectPropDlg::UpdateChapters()
   }
   ctrl->SetSelection(chapter);
   ctrl->Enable(ctrl->GetCount()>1);
+  if (m_videoFrameRadio)
+    m_videoFrameRadio->Enable(GetVideoFilename().length());
 }
 
 void MenuObjectPropDlg::OnJumpActionSelected(wxCommandEvent& WXUNUSED(evt))
@@ -324,16 +379,36 @@ void MenuObjectPropDlg::OnCustomActionSelected(wxCommandEvent& WXUNUSED(evt))
   }
 }
 
-void MenuObjectPropDlg::OnDefaultSizeSelected(wxCommandEvent& WXUNUSED(evt))
+wxString MenuObjectPropDlg::GetVideoFilename(bool withTimestamp, long position)
 {
-  m_widthEdit->Enable(false);
-  m_heightEdit->Enable(false);
+  if (!GetBool(0))
+    return wxT("");
+  if (GetPgci() % 2 == 0)
+    return wxT("");
+  Vob* vob = m_dvd->GetVob(GetTsi(), false, GetPgci()/2, 0);
+  if (!vob)
+	  return wxT("");
+  if (!withTimestamp)
+	  return vob->GetVideoFrameUri(-1);
+  return vob->GetVideoFrameUri(GetChapter(), position);
 }
 
-void MenuObjectPropDlg::OnCustomSizeSelected(wxCommandEvent& WXUNUSED(evt))
-{
-  m_widthEdit->Enable(true);
-  m_heightEdit->Enable(true);
+void MenuObjectPropDlg::OnSelectVideoFrame(wxCommandEvent& evt) {
+	long idx = 0;
+	((wxWindow*)evt.GetEventObject())->GetName().ToLong(&idx);
+	wxWindow* videoFrameSpin = this->FindWindowByName(
+			wxString::Format(wxT("videoFrame_%d"), idx));
+	videoFrameSpin->Enable(evt.GetSelection() == 1);
+}
+
+void MenuObjectPropDlg::OnDefaultSizeSelected(wxCommandEvent& WXUNUSED(evt)) {
+	m_widthEdit->Enable(false);
+	m_heightEdit->Enable(false);
+}
+
+void MenuObjectPropDlg::OnCustomSizeSelected(wxCommandEvent& WXUNUSED(evt)) {
+	m_widthEdit->Enable(true);
+	m_heightEdit->Enable(true);
 }
 
 
@@ -350,7 +425,7 @@ void MenuObjectPropDlg::AddStaticLine(wxSizer* sizer, wxString caption)
 void MenuObjectPropDlg::AddDirectionProp(wxSizer* sizer, ButtonDirCommand dir)
 {
   wxFlexGridSizer* grid = new wxFlexGridSizer(2, 1, 4, 16);
-  AddChoiceProp(grid, wxT(""), wxT(""), wxArrayString(), wxCB_READONLY);
+  AddChoiceProp(grid, wxT(""), wxT(""), wxArrayString());
   wxChoice* ctrl = (wxChoice*) m_controls[m_controls.Count()-1];
   
   // add all buttons except current
@@ -370,6 +445,14 @@ void MenuObjectPropDlg::AddDirectionProp(wxSizer* sizer, ButtonDirCommand dir)
       ctrl->Append(text, new wxString(id));
     }
   }
+  for (int i=0; i<m_menu->GetActionsCount(); i++)
+  {
+    MenuAction* action = m_menu->GetAction(i);
+    wxString id = action->GetId();
+    if (m_object->GetDirection(dir-bdLEFT) == id)
+       sel = ctrl->GetCount();
+    ctrl->Append(id, new wxString(id));
+  }
   ctrl->SetSelection(sel);
   ctrl->SetSizeHints(ctrl->GetSize().GetWidth()*3/2, -1, ctrl->GetSize().GetWidth()*3/2, -1);
   sizer->Add(grid, 0, 0, 0);
@@ -452,12 +535,36 @@ bool MenuObjectPropDlg::SetValues()
         m_object->SetParam(param->name, GetString(n++));
         m_object->SetParamFont(param->name, GetFont(n++).GetChosenFont());
       }
-      else if (param->type == _T("string") || param->type == _T("image"))
+      else if (param->type == _T("string") || param->type == _T("decimal"))
         m_object->SetParam(param->name, GetString(n++));
       else if (param->type.Mid(0,7) == _T("integer"))
         m_object->SetParamInt(param->name, GetInt(n++));
-      else if (param->type == _T("colour"))
+      else if (param->type == _T("colour")) {
         m_object->SetParamColour(param->name, GetColour(n++));
+        wxString opacity = wxString::Format(wxT("%g"), (double)GetInt(n++)/100);
+        m_object->SetParam(param->name, opacity, wxT("-opacity"));
+      }
+      else if (param->type == _T("image"))
+      {
+        if (m_object->IsButton())
+        {
+          bool image = GetBool(n++);
+          wxString imgFile = GetString(n++);
+          n++;
+          if (!image) {
+        	  int videoFrame = -1;
+        	  if (GetInt(n++) == 1) // custom frame
+        		  videoFrame = GetInt(n++);
+        	  else
+        		  n++;
+        	  imgFile = GetVideoFilename(true, videoFrame);
+          } else
+        	  n += 2;
+          m_object->SetParam(param->name, imgFile);
+        }
+        else
+          m_object->SetParam(param->name, GetString(n++));
+      }
     }
   }
   
