@@ -3,7 +3,7 @@
 // Purpose:     Progress of generation/burning dialog
 // Author:      Alex Thuering
 // Created:     14.08.2004
-// RCS-ID:      $Id: ProgressDlg.cpp,v 1.40 2006/12/06 14:31:10 ntalex Exp $
+// RCS-ID:      $Id: ProgressDlg.cpp,v 1.58 2008/04/28 20:54:12 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -16,6 +16,10 @@
 #include <wx/dir.h>
 #include <wx/filename.h>
 #include <wx/file.h>
+#include <wx/regex.h>
+#include <wxSVG/SVGDocument.h>
+#include <wxSVG/mediadec_ffmpeg.h>
+#include "mediaenc_ffmpeg.h"
 
 #define DATA_FILE(fname) wxFindDataFile(_T("data") + wxString(wxFILE_SEP_PATH) + fname)
 
@@ -43,32 +47,103 @@ class ProcessExecute: public wxPipeExecute
 	ProgressDlg* m_processDlg;
 };
 
-class ProcentExecute: public ProcessExecute
+class ProgressExecute: public ProcessExecute
 {
   public:
-	ProcentExecute(ProgressDlg* processDlg):
+	ProgressExecute(ProgressDlg* processDlg, wxString filter):
+	  ProcessExecute(processDlg), m_percent(0)
+	{
+      m_percentPattern.Compile(wxT("(([0-9]+[\\.,][0-9]+)|([0-9]+))%"),wxRE_ICASE);
+      m_blockPattern.Compile(wxT("([0-9]+)[[:space:]]+of[[:space:]]+([0-9]+)"),wxRE_ICASE);
+      m_filterPattern.Compile(filter,wxRE_ICASE);
+	  m_initSubStep = m_processDlg->GetSubStep();
+	}
+	virtual ~ProgressExecute() {};
+	
+	virtual void ProcessOutput(wxString line)
+	{
+      // get last output if program is using \b (remove \b at begin/end, then get text after last \b)
+      while (line.at(0) == wxT('\b')) line.Remove(0,1);
+      while (line.Last() == wxT('\b')) line.RemoveLast(1);
+      line = line.AfterLast(wxT('\b'));
+      if (m_filterPattern.Matches(line))
+      {
+        if (m_blockPattern.Matches(line))
+        {
+		  long blocks = 0;
+		  long totalBlocks = 0;
+		  long percent = 0;
+          if (m_blockPattern.GetMatch(line,1).ToLong(&blocks) && m_blockPattern.GetMatch(line,2).ToLong(&totalBlocks))
+          {
+            percent = (totalBlocks > 0)?(blocks*100)/totalBlocks:0;
+  	        m_processDlg->SetSubStep(m_initSubStep + (int)m_percent);
+		    if (percent >= m_percent)
+            {
+			  m_percent += 5;
+            }
+            else
+            {
+              return;
+            }
+          }
+        }
+        else if (m_percentPattern.Matches(line))
+        {
+		  long percent = 0;
+		  wxString percentStr = m_percentPattern.GetMatch(line,1);
+		  percentStr = percentStr.BeforeFirst(wxT('.')).BeforeFirst(wxT(','));
+		  if (percentStr.ToLong(&percent))
+		  {
+		    m_processDlg->SetSubStep(m_initSubStep + (int)percent);
+		    if (percent >= m_percent)
+            {
+			  m_percent += 5;
+            }
+            else if (percent < m_percent - 5)
+            {
+              m_initSubStep += 100;
+			  m_percent = 5;
+            }
+            else
+            {
+              return;
+            }
+		  }
+        }
+      }
+	  m_processDlg->AddDetailText(line + _T("\n"));
+	}
+
+  protected:
+    wxRegEx m_percentPattern;
+    wxRegEx m_blockPattern;
+    wxRegEx m_filterPattern;
+	int     m_initSubStep;
+	int     m_percent;
+};
+
+class BlocksExecute: public ProcessExecute
+{
+  public:
+	BlocksExecute(ProgressDlg* processDlg):
 	  ProcessExecute(processDlg), m_percent(0)
 	{
 	  m_initSubStep = m_processDlg->GetSubStep();
 	}
-	virtual ~ProcentExecute() {};
+	virtual ~BlocksExecute() {};
 	
 	virtual void ProcessOutput(wxString line)
 	{
-	  if (line.Find(wxT('%'))>0)
-	  {
-		long percent = 0;
-		wxString percentStr = line.BeforeLast(wxT('%')).Strip().AfterLast(wxT(' '));
-		percentStr = percentStr.BeforeFirst(wxT('.')).BeforeFirst(wxT(','));
-		if (percentStr.ToLong(&percent))
-		{
-		  m_processDlg->SetSubStep(m_initSubStep + (int)percent);
-		  if (percent >= m_percent)
-			m_percent += 5;
-		  else
-			return;
-		}
-	  }
+	  long blocks = 0;
+      long totalBlocks = 0;
+      wxRegEx pattern(wxT(".*[[:space:]]+([0-9]+)[[:space:]]+of[[:space:]]+([0-9]+)[[:space:]]+.*"));
+      if (pattern.Matches(line))
+      {
+        pattern.GetMatch(line,1).ToLong(&blocks);
+        pattern.GetMatch(line,2).ToLong(&totalBlocks);
+        m_percent = (totalBlocks > 0)?(blocks*100)/totalBlocks:0;
+	    m_processDlg->SetSubStep(m_initSubStep + (int)m_percent);
+      }
 	  m_processDlg->AddDetailText(line + _T("\n"));
 	}
 
@@ -77,10 +152,10 @@ class ProcentExecute: public ProcessExecute
 	int m_percent;
 };
 
-class DVDAuthorExecute: public ProcentExecute
+class DVDAuthorExecute: public ProgressExecute
 {
   public:
-	DVDAuthorExecute(ProgressDlg* processDlg, int totalSize): ProcentExecute(processDlg),
+	DVDAuthorExecute(ProgressDlg* processDlg, int totalSize): ProgressExecute(processDlg, wxT(".*")),
 	  m_totalSize(totalSize), m_warnings(0), m_warnStep(1), m_dvdauthorStep(0)
 	{
 	  if (m_totalSize == 0)
@@ -91,7 +166,7 @@ class DVDAuthorExecute: public ProcentExecute
 	virtual void ProcessOutput(wxString line)
 	{
 	  if (m_dvdauthorStep)
-		return ProcentExecute::ProcessOutput(line);
+		return ProgressExecute::ProcessOutput(line);
 	  if (line.Mid(0,11) == _T("STAT: fixed"))
 	  {
 		m_dvdauthorStep++;
@@ -310,6 +385,7 @@ void ProgressDlg::Run(BurnDlg* burnDlg, DVD* dvd)
   wxString tmpDir = burnDlg->GetTempDir();
   if (tmpDir.Last() != wxFILE_SEP_PATH)
     tmpDir += wxFILE_SEP_PATH;
+  wxString tmpDVDDir = tmpDir + wxString(wxT("dvd")) + wxFILE_SEP_PATH;
   
   // menus
   wxArrayPtrVoid menuVobs;
@@ -322,18 +398,18 @@ void ProgressDlg::Run(BurnDlg* burnDlg, DVD* dvd)
 	  Pgc* pgc = pgcs[pgci];
 	  if (pgc->GetVobs().Count() == 0)
 	    continue;
-	  wxString menuFile = tmpDir + wxString::Format(_T("menu%d-%d.mpg"), tsi+1, pgci);
+	  wxString menuFile = tmpDVDDir + wxString::Format(_T("menu%d-%d.mpg"), tsi+1, pgci);
 	  Vob* vob = pgc->GetVobs()[0];
 	  vob->SetTmpFilename(menuFile);
-	  menuSubSteps += 10;
+	  menuSubSteps += 10; // perpare
 	  if (vob->GetMenu()->HasVideoBackground())
 	  {
 		if (!MPEG::HasNavPacks(vob->GetMenu()->GetBackground()))
 		  menuSubSteps += 200; // fixvob(200)
-		menuSubSteps += 100; // spumux(100)
 	  }
 	  else
-		menuSubSteps += 150; // jpeg->mpg(50) + mplex(50) + spumux(50)
+        menuSubSteps += 50; // generate mpeg
+      menuSubSteps += 50; // spumux
 	  menuVobs.Add(vob);
 	}
   }
@@ -367,7 +443,7 @@ void ProgressDlg::Run(BurnDlg* burnDlg, DVD* dvd)
 			continue;
 		  titleSubSteps += 100;
 		}
-		vob->SetTmpFilename(tmpDir + wxString::Format(_T("title%d-%d-%d.vob"), tsi, pgci, vobi));
+		vob->SetTmpFilename(tmpDVDDir + wxString::Format(_T("title%d-%d-%d.vob"), tsi, pgci, vobi));
 		titleVobs.Add(vob);
 	  }
 	}
@@ -387,8 +463,8 @@ void ProgressDlg::Run(BurnDlg* burnDlg, DVD* dvd)
 		Vob* vob = pgc->GetVobs()[vobi];
         if (vob->GetSlideshow())
         {
-          slideshowSubSteps += 24*vob->GetSlideshow()->Count();
-          vob->SetTmpFilename(tmpDir + wxString::Format(_T("title%d-%d-%d.vob"), tsi, pgci, vobi));
+          slideshowSubSteps += 10*vob->GetSlideshow()->Count();
+          vob->SetTmpFilename(tmpDVDDir + wxString::Format(_T("title%d-%d-%d.vob"), tsi, pgci, vobi));
           slideshowVobs.Add(vob);
         }
 	  }
@@ -410,7 +486,7 @@ void ProgressDlg::Run(BurnDlg* burnDlg, DVD* dvd)
 		if (vob->GetSubtitles().Count())
         {
           subtitleSubSteps += vob->GetSize()*vob->GetSubtitles().Count();
-          vob->SetTmpFilename(tmpDir + wxString::Format(_T("title%d-%d-%d.vob"), tsi, pgci, vobi));
+          vob->SetTmpFilename(tmpDVDDir + wxString::Format(_T("title%d-%d-%d.vob"), tsi, pgci, vobi));
           subtitleVobs.Add(vob);
 		}
 	  }
@@ -427,10 +503,18 @@ void ProgressDlg::Run(BurnDlg* burnDlg, DVD* dvd)
 	stepCount++;
   if (subtitleVobs.Count()>0)
     stepCount++;
-  if (burnDlg->DoCreateIso() || burnDlg->DoBurn())
+  if (burnDlg->DoCreateIso() || (burnDlg->DoBurn() && burnDlg->DoAddECC()))
+  {
 	stepCount++;
-  if (burnDlg->DoBurn() && burnDlg->DoFormat())
+  }
+  if (burnDlg->DoAddECC())
+    stepCount++;
+  if (burnDlg->DoBurn())
+  {
 	stepCount++;
+	if (burnDlg->DoFormat())
+      stepCount++;
+  }
   SetSteps(stepCount);
   
   // 1. Prepare
@@ -456,6 +540,11 @@ void ProgressDlg::Run(BurnDlg* burnDlg, DVD* dvd)
 	Failed(wxString::Format(_("Can't create directory '%s'"), tmpDir.c_str()));
     return;
   }
+  if (!wxDir::Exists(tmpDVDDir) && !wxMkdir(tmpDVDDir))
+  {
+	Failed(wxString::Format(_("Can't create directory '%s'"), tmpDVDDir.c_str()));
+    return;
+  }
         
   // 2. Generate menus
   if (menuVobs.Count()>0)
@@ -468,7 +557,8 @@ void ProgressDlg::Run(BurnDlg* burnDlg, DVD* dvd)
 	{
 	  AddDetailMsg(wxString::Format(_("Generating menu %d of %d"), i+1, menuVobs.Count()));
 	  Vob* vob = (Vob*)menuVobs[i];
-	  if (!GenerateMenu(vob->GetMenu(), vob->GetTmpFilename(), dvd->GetAudioFormat()))
+	  if (!GenerateMenu(vob->GetMenu(), vob->GetTmpFilename(), dvd->GetAudioFormat(),
+			  vob->GetAudioFilenames().size() > 0 ? vob->GetAudioFilenames()[0] : wxT("")))
 		return;
 	}
 	IncStep();
@@ -551,13 +641,13 @@ void ProgressDlg::Run(BurnDlg* burnDlg, DVD* dvd)
   // 5. Generate dvd filesystem
   if (IsCanceled())
     return;
-  wxString dvdauthFile = tmpDir + _T("dvdauthor.xml");
+  wxString dvdauthFile = tmpDVDDir + _T("dvdauthor.xml");
   dvd->SaveDVDAuthor(dvdauthFile);
   AddSummaryMsg(_("Generating DVD"));
   SetSubSteps(300);
   wxString cmd = s_config.GetDvdauthorCmd();
   cmd.Replace(_T("$FILE_CONF"), dvdauthFile);
-  cmd.Replace(_T("$DIR"), tmpDir);
+  cmd.Replace(_T("$DIR"), tmpDVDDir.Mid(0, tmpDVDDir.length()-1));
   DVDAuthorExecute dvdauthorExec(this, dvd->GetSize(true)/1024);
   if (!dvdauthorExec.Execute(cmd))
   {
@@ -565,15 +655,15 @@ void ProgressDlg::Run(BurnDlg* burnDlg, DVD* dvd)
     return;
   }
   // remove temp files
-  wxDir d(tmpDir);
-  wxString fname;
   if (s_config.GetRemoveTempFiles())
   {
+    wxDir d(tmpDir + wxString(wxT("dvd")));
+    wxString fname;
 	while (d.GetFirst(&fname, _T("*.vob"), wxDIR_FILES))
-	  if (!DeleteFile(tmpDir + fname))
+	  if (!DeleteFile(tmpDVDDir + fname))
 	    return;
 	while (d.GetFirst(&fname, _T("menu*"), wxDIR_FILES))
-	  if (!DeleteFile(tmpDir + fname))
+	  if (!DeleteFile(tmpDVDDir + fname))
 	    return;
 	if (!DeleteFile(dvdauthFile))
 	    return;
@@ -581,31 +671,35 @@ void ProgressDlg::Run(BurnDlg* burnDlg, DVD* dvd)
   IncStep();
   
   // 6. Preview
-  if (burnDlg->DoPreview())
-  {
+  if (burnDlg->DoPreview()) {
     if (IsCanceled())
       return;
     AddSummaryMsg(_("Start preview"));
     wxString cmd = s_config.GetPreviewCmd();
-    cmd.Replace(_T("$DIR"), tmpDir);
-    if (!Exec(cmd))
-      FailedExec(cmd);
-    if (burnDlg->DoBurn() && wxMessageBox(_("Burn this video to dvd?"),
-         _("Burn"), wxYES_NO|wxICON_QUESTION, this) == wxNO)
-	{
-	  AddSummaryMsg(_("Aborted"), wxEmptyString, *wxRED);
-      return;
-	}
-    if (burnDlg->DoCreateIso() && wxMessageBox(_("Create iso image of this video?"),
-         _("Burn"), wxYES_NO|wxICON_QUESTION, this) == wxNO)
-    {
-	  AddSummaryMsg(_("Aborted"), wxEmptyString, *wxRED);
-      return;
-	}
+    if (cmd.length() > 0) {
+    	cmd.Replace(_T("$DIR"), tmpDVDDir.Mid(0, tmpDVDDir.length()-1));
+	    if (!Exec(cmd))
+	      FailedExec(cmd);
+    } else {
+    	wxString msg = _("Unfortunately there is no DVD player specified in the DVDStyler settings. \
+Please set the path to the DVD player in the 'Settings/Core/Preview command' \
+or open the following directory with your DVD player: ");
+    	msg += tmpDVDDir;
+    	wxMessageBox(msg, _("Burn"), wxOK, this);
+    }
+    if (burnDlg->DoBurn() || burnDlg->DoCreateIso()) {
+	    wxString msg = burnDlg->DoBurn() ?
+	    		_("Do you want to burn this video to DVD?") :
+	    		_("Do you want to create an iso image of this video?");
+	    if (wxMessageBox(msg, _("Burn"), wxYES_NO|wxICON_QUESTION, this) == wxNO) {
+	    	AddSummaryMsg(_("Aborted"), wxEmptyString, *wxRED);
+	    	return;
+		}
+    }
   }
   
   // 7. Create ISO-Image
-  if (burnDlg->DoCreateIso())
+  if (burnDlg->DoCreateIso() || (burnDlg->DoBurn() && burnDlg->DoAddECC()))
   {
     if (IsCanceled())
       return;
@@ -613,25 +707,41 @@ void ProgressDlg::Run(BurnDlg* burnDlg, DVD* dvd)
 	SetSubSteps(100);
     wxString cmd = s_config.GetIsoCmd();
     cmd.Replace(_T("$VOL_ID"), dvd->GetName());
-    cmd.Replace(_T("$DIR"), tmpDir);
-    cmd.Replace(_T("$FILE"), burnDlg->GetIsoFile());
-	ProcentExecute exec(this);
+    cmd.Replace(_T("$DIR"), tmpDVDDir.Mid(0, tmpDVDDir.length()-1));
+    cmd.Replace(_T("$FILE"), burnDlg->DoCreateIso() ? burnDlg->GetIsoFile() : tmpDir + wxT("dvd.iso"));
+	ProgressExecute exec(this, wxT(".*"));
     if (!exec.Execute(cmd))
     {
       FailedExec(cmd);
       return;
     }
-    // remove tmp dir
-  	DeleteTempDir(tmpDir);
 	IncStep();
   }
   
-  // 8. Format DVD
+  // 8. Add ECC-data
+  if (burnDlg->DoAddECC())
+  {
+    if (IsCanceled())
+      return;
+    AddSummaryMsg(_("Adding ECC data"));
+	SetSubSteps(200);
+    wxString cmd = s_config.GetAddECCCmd();
+    cmd.Replace(_T("$FILE"), burnDlg->DoCreateIso() ? burnDlg->GetIsoFile() : tmpDir + wxT("dvd.iso"));
+	ProgressExecute exec(this,wxT("(Preparing|Ecc).*"));
+    if (!exec.Execute(cmd))
+    {
+      FailedExec(cmd);
+      return;
+    }
+	IncStep();
+  }
+
+  // 9. Format DVD
   if (burnDlg->DoBurn() && burnDlg->DoFormat())
   {
     if (IsCanceled())
       return;
-    AddSummaryMsg(_("Formating DVD-RW"));
+    AddSummaryMsg(_("Formatting DVD-RW"));
     while (1)
     {
 	  SetSubSteps(100);
@@ -639,7 +749,7 @@ void ProgressDlg::Run(BurnDlg* burnDlg, DVD* dvd)
       cmd.Replace(_T("$DEV"), burnDlg->GetDevice());
       if (!Exec(cmd))
       {
-	    int repeat = wxMessageBox(_("Formating DVD-RW failed. Try again?"),
+	    int repeat = wxMessageBox(_("Formatting DVD-RW failed. Try again?"),
 	      _("Burn"), wxYES_NO|wxCANCEL|wxICON_QUESTION, this);
         if (repeat == wxYES)
 	      continue;
@@ -659,68 +769,115 @@ void ProgressDlg::Run(BurnDlg* burnDlg, DVD* dvd)
 	IncStep();
   }
   
-  // 9. Burn DVD
+  // 10. Burn DVD
   if (burnDlg->DoBurn())
   {
     if (IsCanceled())
       return;
     AddSummaryMsg(_("Burning"));
 	SetSubSteps(100);
-	// burn
-    cmd = s_config.GetBurnCmd();
-    cmd.Replace(_T("$VOL_ID"), dvd->GetName());
-    cmd.Replace(_T("$DIR"), tmpDir);
-    cmd.Replace(_T("$DEV"), burnDlg->GetDevice());
+	// check disc
+	cmd = s_config.GetBurnScanCmd();
+    cmd.Replace(_T("$DEVICE"), burnDlg->GetDevice());
 #ifdef __WXMSW__
-    // get iso size
-    if (s_config.GetIsoSizeCmd().length())
-    {
-	  wxString cmd = s_config.GetIsoSizeCmd();
-      cmd.Replace(_T("$DIR"), tmpDir);
-      cmd = wxGetAppPath() + wxString(wxFILE_SEP_PATH) + _T("..") +
-        wxString(wxFILE_SEP_PATH) + _T("dvdauthor") +
-        wxString(wxFILE_SEP_PATH) + cmd;
+    cmd = wxGetAppPath() + wxString(wxFILE_SEP_PATH) + _T("..") + wxString(wxFILE_SEP_PATH)
+      + _T("dvdauthor") + wxString(wxFILE_SEP_PATH) + cmd;
+#endif
+    wxArrayString output;
+    while (true) {
+    	if (wxExecute(cmd, output, wxEXEC_SYNC | wxEXEC_NODISABLE) == 0)
+    		break;
+    	if (wxMessageBox(wxString::Format(
+    			_("Please insert empty DVD into the device %s."), burnDlg->GetDevice().c_str()),
+    			_("Burn"), wxOK|wxCANCEL, this) == wxCANCEL) {
+    		AddSummaryMsg(_("Aborted"), wxEmptyString, *wxRED);
+    		return;
+    	}
+    }
+	// get disc size
+	long discSize = 0;
+	for (unsigned int i = 0; i < output.Count(); i++)
+      if (output[i].length() > 12 && output[i].SubString(1,12) == wxT("Free Blocks:")) {
+        wxString discSizeStr = output[i].AfterFirst(wxT(':')).Trim(false).BeforeFirst(wxT('*'));
+        discSizeStr.ToLong(&discSize);
+        AddDetailMsg(wxString::Format(wxT("Disc size: %d MB"), discSize/512));
+        break;
+      }
+	if (discSize < 2290000)
+	  discSize = 2295104;
+	// check size
+	long size = 0;
+	if (burnDlg->DoAddECC()) {
+      size = wxFile(tmpDir + wxT("dvd.iso")).Length();
+    } else {
+      cmd = s_config.GetIsoSizeCmd();
+      cmd.Replace(_T("$DIR"), tmpDVDDir.Mid(0, tmpDVDDir.length()-1));
+#ifdef __WXMSW__
+        cmd = wxGetAppPath() + wxString(wxFILE_SEP_PATH) + _T("..") + wxString(wxFILE_SEP_PATH)
+          + _T("dvdauthor") + wxString(wxFILE_SEP_PATH) + cmd;
+#endif
       wxArrayString output;
-      if (wxExecute(cmd, output, wxEXEC_SYNC) != 0)
-      {
-        FailedExec(cmd);
-        return;
-      }
-      long size = 0;
-      if (!output.Count() || !output[0].ToLong(&size))
-      {
-        Failed(wxT("Failed getting size of DVD"));
-        return;
+      wxExecute(cmd, output, wxEXEC_SYNC | wxEXEC_NODISABLE);
+      if (output.Count() > 0 && output[0].length() > 0) {
+        output[0].ToLong(&size);
+        size = (size + 254) * 2048;
       }
+    }
+    AddDetailMsg(wxString::Format(wxT("ISO Size: %d MB"), size/1024/1024));
+    if (size > discSize*2048
+        && wxMessageBox(wxString::Format(
+          _("Size of Disc Image > %.2f GB. Do you want to continue?"), (double)discSize/512/1024),
+          _("Burn"), wxYES_NO|wxICON_QUESTION, this) == wxNO)
+    {
+      AddSummaryMsg(_("Aborted"), wxEmptyString, *wxRED);
+	  return;
+    }
+	// burn
+	if (burnDlg->DoAddECC())
+    {
+      cmd = s_config.GetBurnISOCmd();
+      cmd.Replace(_T("$FILE"), tmpDir + wxT("dvd.iso"));
+      // get iso size in sectors
+      long size = wxFile(tmpDir + wxT("dvd.iso")).Length()/2048;
       cmd.Replace(_T("$SIZE"), wxString::Format(wxT("%d"),size));
     }
-#endif
-	wxString speedStr;
+    else
+    {
+      cmd = s_config.GetBurnCmd();
+      cmd.Replace(_T("$DIR"), tmpDVDDir.Mid(0, tmpDVDDir.length()-1));
+    }
+    cmd.Replace(_T("$VOL_ID"), dvd->GetName());
+    cmd.Replace(_T("$DEV"), burnDlg->GetDevice());
+    wxString speedStr;
 	if (burnDlg->GetSpeed() > 0)
 	{
 	  speedStr = s_config.GetBurnSpeedOpt();
       speedStr.Replace(_T("$SPEED"), wxString::Format(_T("%d"), burnDlg->GetSpeed()));
     }
 	cmd.Replace(_T("$SPEEDSTR"), speedStr);
-	ProcentExecute exec(this);
+	ProgressExecute exec(this, wxT(".*"));
     if (!exec.Execute(cmd))
     {
       FailedExec(cmd);
       return;
     }
-    // remove tmp tmpDir
-    DeleteTempDir(tmpDir);
 	IncStep();
   }
   
   if (IsCanceled())
     return;
   
+  // remove tmp dir
+  if ((burnDlg->DoCreateIso() || burnDlg->DoBurn()) && s_config.GetRemoveTempFiles())
+    DeleteTempDir(tmpDir);
+  
   if (burnDlg->DoBurn())
     AddSummaryMsg(_("Burning was successful."), wxEmptyString, wxColour(0,128,0));
   else
     AddSummaryMsg(_("Generating was successful."), wxEmptyString, wxColour(0,128,0));
   IncStep();
+  
+  s_config.SetKey(s_config.GetKey()+1);
 }
 
 bool ProgressDlg::MakeVob(const wxString& videoFile,
@@ -763,7 +920,7 @@ bool ProgressDlg::FixVob(const wxString& mpegFile,
   wxString cmd = s_config.GetDemplexCmd();
   cmd.Replace(_T("$FILE_IN"), mpegFile);
   cmd.Replace(_T("$FILES_OUT"), tmpDir + tmpFile);
-  ProcentExecute exec(this);
+  ProgressExecute exec(this,wxT(".*"));
   if (!exec.Execute(cmd))
   {
 	FailedExec(cmd);
@@ -791,13 +948,12 @@ bool ProgressDlg::FixVob(const wxString& mpegFile,
   return true;
 }
 
-bool ProgressDlg::GenerateMenu(Menu* menu, const wxString& menuFile, AudioFormat audioFormat)
-{
+bool ProgressDlg::GenerateMenu(Menu* menu, const wxString& menuFile,
+		AudioFormat audioFormat, wxString audioFile) {
   if (IsCanceled())
 	return false;
-  wxString bgFile  = menuFile + _T("_bg.jpg");
-  wxString bgvFile = menuFile + _T("_bg.m2v");
-  wxString bgmFile = menuFile + _T("_bg.mpg");
+  wxString mpegFile = menuFile + _T("_bg.mpg");
+  wxString m2vFile = menuFile + _T("_bg.m2v");
   wxString btFile  = menuFile + _T("_buttons.png");
   wxString hlFile  = menuFile + _T("_highlight.png");
   wxString selFile = menuFile + _T("_select.png");
@@ -812,11 +968,7 @@ bool ProgressDlg::GenerateMenu(Menu* menu, const wxString& menuFile, AudioFormat
   AddDetailMsg(_("Prepare"));
   menu->SetTransparentColor();
   wxImage* images = menu->GetImages();
-  if (!videoMenu)
-  {
-    images[0].SetOption(_T("quality"), 100);
-    images[0].SaveFile(bgFile);
-  }
+  wxImage bgImage = images[0];
   images[1].SaveFile(btFile);
   images[2].SaveFile(hlFile);
   images[3].SaveFile(selFile);
@@ -826,65 +978,73 @@ bool ProgressDlg::GenerateMenu(Menu* menu, const wxString& menuFile, AudioFormat
   IncSubStep(10);
   
   // convert jpeg to mpeg
-  if (videoMenu)
-  {
-	if (!MPEG::HasNavPacks(menu->GetBackground()))
+  AddDetailMsg(_("Create menu mpeg"));
+  if (s_config.GetMenuFrameCount()<25)
+    s_config.SetMenuFrameCount(25);
+  if (s_config.GetMenuVideoBitrate()<1000)
+    s_config.SetMenuVideoBitrate(1000);
+    
+  if (!bgImage.Ok()) {
+    Failed(_("Error creation of menu"));
+    return false;
+  }
+  
+  if (videoMenu) {
+    if (!MPEG::HasNavPacks(menu->GetBackground()))
 	{
 	  int subStep = GetSubStep();
       wxArrayString audioFiles;
-	  if (!FixVob(menu->GetBackground(), audioFiles, bgmFile))
+	  if (!FixVob(menu->GetBackground(), audioFiles, mpegFile))
 		return false;
 	  SetSubStep(subStep+200);
 	}
 	else
-	  bgmFile = menu->GetBackground();
-  }
-  else
-  {
-    AddDetailMsg(_("Converting jpg to mpeg"));
-    wxString videoNorm;
-    wxString frameRate = wxT("25");
-    if (menu->GetVideoFormat() == vfNTSC)
-    {
-	  videoNorm = wxT(" --video-norm n --frame-rate 4");
-	  frameRate = wxT("29.97");
-    }
-    wxString cmd = s_config.GetJpeg2MpegCmd();
-    cmd.Replace(_T("$FRAME_RATE"), frameRate);
-    cmd.Replace(_T("$VIDEO_NORM"), videoNorm);
-	cmd.Replace(_T("$FRAMES"), wxString::Format(_T("%d"),
-      s_config.GetMenuFrameCount()));
-	cmd.Replace(_T("$BITRATE"),
-	  wxString::Format(_T("%d"), s_config.GetMenuVideoBitrate()));
-    cmd.Replace(_T("$FILE_IN"), bgFile);
-    cmd.Replace(_T("$FILE_OUT"), bgvFile);
-    long ret = Exec(cmd);
-    if (!ret)
-    {
-	  FailedExec(cmd);
-	  return false;
-    }
-	if (s_config.GetRemoveTempFiles())
-	  if (!DeleteFile(bgFile))
-	    return false;
-	IncSubStep(50);
-    
-    if (IsCanceled())
+	  mpegFile = menu->GetBackground();
+  } else {
+	  int frameCount = s_config.GetMenuFrameCount();
+	  if (audioFile.length() > 0) {
+		  wxFfmpegMediaDecoder decoder;
+		  if (decoder.Load(audioFile)) {
+			  double duration = decoder.GetDuration();
+			  if (duration > 0) {
+				  AddDetailMsg(wxString::Format(_("Audio duration: %f sec"), duration));
+				  if (menu->GetVideoFormat() == vfPAL)
+					  frameCount = (int) (duration * 25);
+				  else
+					  frameCount = (int) (duration * 30000/1001);
+			  }
+		  }
+	  } else
+		  audioFile = silenceFile;
+	  wxFfmpegMediaEncoder ffmpeg;
+	  if (!ffmpeg.BeginEncode(audioFile.length() ? m2vFile : mpegFile,
+			  menu->GetVideoFormat(), audioFile.length() ? afNONE : audioFormat,
+					  s_config.GetMenuVideoBitrate())
+		  || !ffmpeg.EncodeImage(bgImage, frameCount)) {
+      Failed(_("Error creation of menu"));
       return false;
-    AddDetailMsg(_("Multiplexing audio and video"));
-  	cmd = s_config.GetMplexCmd();
-    cmd.Replace(_T("$FILE_VIDEO"), bgvFile);
-    cmd.Replace(_T("$FILE_AUDIO"), silenceFile);
-    cmd.Replace(_T("$FILE_OUT"), bgmFile);
-    if (!Exec(cmd))
-    {
-	  FailedExec(cmd);
-	  return false;
     }
-	if (s_config.GetRemoveTempFiles())
-	  if (!DeleteFile(bgvFile))
-	    return false;
-	IncSubStep(50);
+    ffmpeg.EndEncode();
+    IncSubStep(25);
+    
+    if (audioFile.length()) {
+    	// mplex
+		if (IsCanceled())
+			return false;
+		AddDetailMsg(_("Multiplexing audio and video"));
+		wxString cmd = s_config.GetMplexCmd();
+		cmd.Replace(_T("$FILE_VIDEO"), m2vFile);
+		cmd.Replace(_T("$FILE_AUDIO"), audioFile);
+		cmd.Replace(_T("$FILE_OUT"), mpegFile);
+		if (!Exec(cmd)) {
+			FailedExec(cmd);
+			return false;
+		}
+		if (s_config.GetRemoveTempFiles())
+			if (!DeleteFile(m2vFile))
+				return false;
+	}
+    IncSubStep(25);
   }
   
   //spumux
@@ -893,15 +1053,15 @@ bool ProgressDlg::GenerateMenu(Menu* menu, const wxString& menuFile, AudioFormat
   AddDetailMsg(_("Multiplexing subtitles (buttons) into mpeg"));
   wxString cmd = s_config.GetSpumuxCmd();
   cmd.Replace(_T("$FILE_CONF"), spuFile);
-  if (!Exec(cmd, bgmFile, menuFile))
+  if (!Exec(cmd, mpegFile, menuFile))
   {
     FailedExec(cmd);
     return false;
   }
   if (s_config.GetRemoveTempFiles())
   {
-	if ((!videoMenu || bgmFile != menu->GetBackground()) &&
-		!DeleteFile(bgmFile))
+	if ((!videoMenu || mpegFile != menu->GetBackground()) &&
+		!DeleteFile(mpegFile))
       return false;
 	if (!DeleteFile(btFile) ||
         !DeleteFile(hlFile) ||
@@ -909,7 +1069,7 @@ bool ProgressDlg::GenerateMenu(Menu* menu, const wxString& menuFile, AudioFormat
         !DeleteFile(spuFile))
       return false;
   }
-  IncSubStep(videoMenu ? 100 : 50);
+  IncSubStep(50);
   
   wxYield();
   return true;
@@ -918,107 +1078,31 @@ bool ProgressDlg::GenerateMenu(Menu* menu, const wxString& menuFile, AudioFormat
 bool ProgressDlg::GenerateSlideshow(Slideshow* slideshow, const wxString& vobFile,
 	AudioFormat audioFormat)
 {
-  if (IsCanceled())
-	return false;
-  AddDetailMsg(_("Generate slideshow"));
-  wxString tmpFile = vobFile.BeforeLast(wxT('.'));
-  
-  wxString jpgFile = tmpFile + _T(".jpg");
-  wxString m2vFile = tmpFile + _T(".m2v");
-  wxString m2vTmpFile = tmpFile + _T("_tmp.m2v");
-  wxString mp2File = tmpFile + _T(".mp2");
-  wxString silenceFile = audioFormat == afMP2 ?
-    DATA_FILE(_T("silence.mp2")) : DATA_FILE(_T("silence.ac3"));
-  
-  wxYield();
-  
-  for (unsigned i=0; i<slideshow->Count(); i++)
-  {
-    // save background
-    AddDetailMsg(wxString::Format(_("Save slide %d image"),i));
-    wxImage img = slideshow->GetImage(i);
-    img.SetOption(_T("quality"), 100);
-    img.SaveFile(jpgFile);
-    IncSubStep(1);
-    
-    // convert jpeg to mpeg
-    AddDetailMsg(wxString::Format(_("Converting slide %d image to video"),i));
-    wxString videoNorm;
-    wxString frameRate = wxT("25");
-    if (slideshow->GetVideoFormat() == vfNTSC)
-    {
-      videoNorm = wxT(" --video-norm n --frame-rate 4");
-      frameRate = wxT("29.97");
-    }
-    wxString cmd = s_config.GetJpeg2MpegCmd();
-    cmd.Replace(_T("$FRAME_RATE"), frameRate);
-    cmd.Replace(_T("$VIDEO_NORM"), videoNorm);
-    cmd.Replace(_T("$FRAMES"), wxString::Format(_T("%d"),
-      (int)(slideshow->GetDuration()*slideshow->GetFPS())));
-    cmd.Replace(_T("$BITRATE"),
-      wxString::Format(_T("%d"), s_config.GetSlideshowVideoBitrate()));
-    cmd.Replace(_T("$FILE_IN"), jpgFile);
-    cmd.Replace(_T("$FILE_OUT"), i == 0 ? m2vFile : m2vTmpFile);
-    long ret = Exec(cmd);
-    if (!ret)
-    {
-      FailedExec(cmd);
-      return false;
-    }
-    if (s_config.GetRemoveTempFiles())
-      if (!DeleteFile(jpgFile))
-        return false;
-    if (i>0)
-    {
-      wxFile src(m2vTmpFile, wxFile::read);
-      wxFile dst(m2vFile, wxFile::write_append);
-      char buf[4096];
-      while (!src.Eof())
-      {
-        int count = src.Read(buf, 4096);
-        dst.Write(buf, count);
-      }
-      src.Close();
-      if (!DeleteFile(m2vTmpFile))
-        return false;
-    }
-    IncSubStep(20);
-  }
-  
-  // create silence mp2
-  AddDetailMsg(_("Create silence audio file"));
-  wxString cmd = s_config.GetSilenceCmd();
-  cmd.Replace(_T("$DURATION"), wxString::Format(_T("%d"), slideshow->GetDuration()*slideshow->Count()));
-  cmd.Replace(_T("$FILE_IN"), silenceFile);
-  cmd.Replace(_T("$FILE_OUT"), mp2File);
-  long ret = Exec(cmd);
-  if (!ret)
-  {
-    FailedExec(cmd);
-    return false;
-  }
-  IncSubStep(slideshow->Count());
-  
-  if (IsCanceled())
-    return false;
-  AddDetailMsg(_("Multiplexing audio and video"));
-  cmd = s_config.GetMplexCmd();
-  cmd.Replace(_T("$FILE_VIDEO"), m2vFile);
-  cmd.Replace(_T("$FILE_AUDIO"), mp2File);
-  cmd.Replace(_T("$FILE_OUT"), vobFile);
-  if (!Exec(cmd))
-  {
-    FailedExec(cmd);
-    return false;
-  }
-  if (s_config.GetRemoveTempFiles() &&
-      !DeleteFile(m2vFile) ||
-      !DeleteFile(mp2File))
-    return false;
-  IncSubStep(2*slideshow->Count());
-  
-  wxYield();
-  return true;
+	if (IsCanceled())
+		return false;
+	AddDetailMsg(_("Generating slideshow"));
+	
+	wxFfmpegMediaEncoder ffmpeg;
+	if (!ffmpeg.BeginEncode(vobFile, slideshow->GetVideoFormat(), audioFormat,
+			s_config.GetSlideshowVideoBitrate())) {
+		Failed(_("Error creation of slideshow"));
+		return false;
+	}
+	
+	for (unsigned i=0; i<slideshow->Count(); i++) {
+		AddDetailMsg(wxString::Format(_("Converting slide %d image to video"),i+1));
+		wxYield();
+		wxImage img = slideshow->GetImage(i);
+		if (!ffmpeg.EncodeImage(img, (int)(slideshow->GetDuration()*slideshow->GetFPS()))) {
+			Failed(_("Error creation of slideshow"));
+			return false;
+		}
+		IncSubStep(10);
+	}
+	ffmpeg.EndEncode();
+	
+	wxYield();
+	return true;
 }
 
 bool ProgressDlg::MultiplexSubtitles(const wxString& vobFile, TextSub* textSub,
@@ -1080,13 +1164,13 @@ bool ProgressDlg::DeleteTempDir(wxString dir)
 {
   if (dir.Last() != wxFILE_SEP_PATH)
 	dir += wxFILE_SEP_PATH;
-  wxDir d(dir);
+  wxDir d(dir + wxT("dvd"));
   wxString fname;
   d.GetFirst(&fname, wxEmptyString, wxDIR_DIRS);
   while (1)
   {
 	if (fname.Lower() == _T("video_ts") || fname.Lower() == _T("audio_ts"))
-	  if (!DeleteDir(dir + fname, false))
+	  if (!DeleteDir(dir + wxT("dvd") + wxFILE_SEP_PATH + fname, false))
         return false;
 	if (!d.GetNext(&fname))
 	  break;
@@ -1096,13 +1180,16 @@ bool ProgressDlg::DeleteTempDir(wxString dir)
   {
 	if (fname.Mid(0,4).Lower() == _T("menu") ||
 		fname.Mid(0,5).Lower() == _T("title"))
-	  if (!DeleteFile(dir + fname))
+	  if (!DeleteFile(dir + wxT("dvd") + wxFILE_SEP_PATH + fname))
 	    return false;
 	if (!d.GetNext(&fname))
 	  break;
   }
+  if (wxFileName(dir + wxT("dvd.iso")).FileExists() && DeleteFile(dir + wxT("dvd.iso")))
+      return false;
   d.Open(wxGetHomeDir());
   wxLogNull log;
+  wxRmdir(dir + wxT("dvd"));
   wxRmdir(dir);
   return true;
 }
