@@ -3,7 +3,7 @@
 // Purpose:     Titleset Manager
 // Author:      Alex Thuering
 // Created:	27.01.2003
-// RCS-ID:      $Id: TitlesetManager.cpp,v 1.11 2007/03/08 20:23:14 ntalex Exp $
+// RCS-ID:      $Id: TitlesetManager.cpp,v 1.16 2008/04/21 20:20:13 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     GPL
 /////////////////////////////////////////////////////////////////////////////
@@ -13,11 +13,16 @@
 #include "MenuPropDlg.h"
 #include "MPEG.h"
 #include "Config.h"
+#include "Utils.h"
 #include <wxVillaLib/ThumbnailFactory.h>
 #include <wxVillaLib/ImageProc.h>
+#include <wxVillaLib/utils.h>
 #include <wx/splitter.h>
 #include <wx/dnd.h>
+#include <wx/clipbrd.h>
 #include <wx/utils.h>
+#include <wx/artprov.h>
+#include "rc/noaudio.png.h"
 
 //////////////////////////////////////////////////////////////////////////////
 ///////////////////////////// TitleDnDFile ///////////////////////////////////
@@ -115,31 +120,35 @@ class TitleDnDFile : public wxFileDropTarget
 /////////////////////////// TitlesetManager //////////////////////////////////
 //////////////////////////////////////////////////////////////////////////////
 
-enum
-{
-  MENU_TITLE_DEL_ID = 2650,
-  MENU_TITLE_PROP_ID,
-  MENU_TITLE_ADD_ID,
-  MENU_TITLE_MOVE_ID,
-  MENU_MAIN_ADD_ID,
-  MENU_ADD_MENU_ID,
-  MENU_ADD_VMMENU_ID,
-  MENU_ADD_TITLESET_ID,
-  MENU_MOVE_TITLE_LEFT_ID,
-  MENU_MOVE_TITLE_RIGHT_ID
+enum {
+	MENU_TITLE_DEL_ID = 2650,
+	MENU_TITLE_COPY_ID,
+	MENU_TITLE_PASTE_ID,
+	MENU_TITLE_PROP_ID,
+	MENU_TITLE_ADD_ID,
+	MENU_TITLE_MOVE_ID,
+	MENU_MAIN_ADD_ID,
+	MENU_ADD_MENU_ID,
+	MENU_ADD_VMMENU_ID,
+	MENU_ADD_TITLESET_ID,
+	MENU_MOVE_TITLE_LEFT_ID,
+	MENU_MOVE_TITLE_RIGHT_ID
 };
 
 BEGIN_EVENT_TABLE(TitlesetManager, wxThumbnails)
-  EVT_MENU(MENU_ADD_MENU_ID, TitlesetManager::OnAddMenu)
-  EVT_MENU(MENU_ADD_VMMENU_ID, TitlesetManager::OnAddVmMenu)
-  EVT_MENU(MENU_ADD_TITLESET_ID, TitlesetManager::OnAddTitleset)
-  EVT_UPDATE_UI(MENU_ADD_TITLESET_ID, TitlesetManager::OnAddTitlesetUpdateUI)
-  EVT_MENU(MENU_MOVE_TITLE_LEFT_ID, TitlesetManager::OnMoveTitleLeft)
-  EVT_MENU(MENU_MOVE_TITLE_RIGHT_ID, TitlesetManager::OnMoveTitleRight)
-  EVT_UPDATE_UI(MENU_MOVE_TITLE_LEFT_ID, TitlesetManager::OnMoveTitleLeftUpdateUI)
-  EVT_UPDATE_UI(MENU_MOVE_TITLE_RIGHT_ID, TitlesetManager::OnMoveTitleRightUpdateUI)
-  EVT_MENU(MENU_TITLE_DEL_ID, TitlesetManager::OnDelete)
-  EVT_MENU(MENU_TITLE_PROP_ID, TitlesetManager::OnProps)
+	EVT_MENU(MENU_ADD_MENU_ID, TitlesetManager::OnAddMenu)
+	EVT_MENU(MENU_ADD_VMMENU_ID, TitlesetManager::OnAddVmMenu)
+	EVT_MENU(MENU_ADD_TITLESET_ID, TitlesetManager::OnAddTitleset)
+	EVT_UPDATE_UI(MENU_ADD_TITLESET_ID, TitlesetManager::OnAddTitlesetUpdateUI)
+	EVT_MENU(MENU_MOVE_TITLE_LEFT_ID, TitlesetManager::OnMoveTitleLeft)
+	EVT_MENU(MENU_MOVE_TITLE_RIGHT_ID, TitlesetManager::OnMoveTitleRight)
+	EVT_UPDATE_UI(MENU_MOVE_TITLE_LEFT_ID, TitlesetManager::OnMoveTitleLeftUpdateUI)
+	EVT_UPDATE_UI(MENU_MOVE_TITLE_RIGHT_ID, TitlesetManager::OnMoveTitleRightUpdateUI)
+	EVT_MENU(MENU_TITLE_DEL_ID, TitlesetManager::OnDelete)
+	EVT_MENU(MENU_TITLE_COPY_ID, TitlesetManager::OnCopy)
+	EVT_MENU(MENU_TITLE_PASTE_ID, TitlesetManager::OnPaste)
+	EVT_UPDATE_UI(MENU_TITLE_PASTE_ID, TitlesetManager::OnUpdateUIObjectPaste)
+	EVT_MENU(MENU_TITLE_PROP_ID, TitlesetManager::OnProps)
 END_EVENT_TABLE()
 
 DEFINE_EVENT_TYPE(EVT_COMMAND_MENU_SELECTED)
@@ -166,10 +175,16 @@ TitlesetManager::TitlesetManager(wxWindow* parent, int id):
   
   m_mainMenu = new wxMenu;
   m_mainMenu->Append(MENU_MAIN_ADD_ID, _("&Add"), addMenu);
+  wxMenuItem* item = new wxMenuItem(m_mainMenu, MENU_TITLE_PASTE_ID, _("&Paste"));
+  item->SetBitmap(wxArtProvider::GetBitmap(wxART_PASTE, wxART_MENU));
+  m_mainMenu->Append(item);
   
   m_titleMenu = new wxMenu;
   m_titleMenu->Append(MENU_TITLE_ADD_ID, _("&Add"), addMenu);
   m_titleMenu->Append(MENU_TITLE_MOVE_ID, _("&Move"), moveMenu);
+  item = new wxMenuItem(m_titleMenu, MENU_TITLE_COPY_ID, _("&Copy"));
+  item->SetBitmap(wxArtProvider::GetBitmap(wxART_COPY, wxART_MENU));
+  m_titleMenu->Append(item);
   m_titleMenu->Append(MENU_TITLE_DEL_ID, _("&Delete"));
   m_titleMenu->Append(MENU_TITLE_PROP_ID, _("&Properties..."));
   
@@ -270,6 +285,24 @@ void TitlesetManager::DrawThumbnail(wxBitmap& bmp, wxThumb& thumb, int index)
 	y + titleHeight + (m_tHeight-titleHeight-img.GetHeight())/2,
     img.GetWidth(), img.GetHeight());      
   dc.DrawBitmap(img, imgRect.x, imgRect.y);
+  // check audio
+  if (vob->GetFilename().length() == 0 && vob->GetVideoFilename().length() > 0
+		  && vob->GetAudioFilenames().GetCount() == 0) {
+	  wxBitmap noaudioBitmap = wxBITMAP_FROM_MEMORY(noaudio);
+	  dc.DrawBitmap(noaudioBitmap,
+			  imgRect.x + img.GetWidth() - noaudioBitmap.GetWidth(),
+			  imgRect.y + img.GetHeight() - noaudioBitmap.GetHeight());
+  } else if (vob->GetAudioFilenames().GetCount() > 0) {
+	  wxString text = vob->GetAudioFilenames()[0].AfterLast(wxT('.')).Upper();
+	  for (int fi = 1; fi < vob->GetAudioFilenames().GetCount(); fi++)
+		  text += wxT(",") + vob->GetAudioFilenames()[fi].AfterLast(wxT('.')).Upper();
+	  dc.SetTextForeground(*wxWHITE);
+	  wxFont font(9, wxFONTFAMILY_ROMAN, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL);
+	  dc.SetFont(font);
+	  wxCoord w, h;
+	  dc.GetTextExtent(text, &w, &h);
+	  dc.DrawText(text, imgRect.x + img.GetWidth() - w - 4, imgRect.y + img.GetHeight() - h);
+  }
   // outline
   if (index == m_selectedMenu)
   {
@@ -278,6 +311,8 @@ void TitlesetManager::DrawThumbnail(wxBitmap& bmp, wxThumb& thumb, int index)
 	dc.DrawRectangle(imgRect.x-1, imgRect.y-1, imgRect.width+1, imgRect.height+1);
   }
   // draw title caption
+  wxFont font(12, wxFONTFAMILY_ROMAN, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL);
+  dc.SetFont(font);
   dc.SetTextForeground(*wxWHITE);
   wxString caption;
   if (isMenu && tsi<0)
@@ -400,7 +435,7 @@ int TitlesetManager::GetIndex(int id)
   return -1;
 }
 
-void TitlesetManager::AddVideo(const wxString& fname, bool createTitle)
+bool TitlesetManager::AddVideo(const wxString& fname, bool createTitle)
 {
   if (m_dvd->GetTitlesets().Count() == 0)
     m_dvd->GetTitlesets().Add(new Titleset);
@@ -413,7 +448,7 @@ void TitlesetManager::AddVideo(const wxString& fname, bool createTitle)
   {
     wxLogError(wxString(_("The last title isn't complete.\n")) +
                         _("Please add an audio track to it."));
-    return;
+    return false;
   }
     
   vob = new Vob(_T(""));
@@ -421,8 +456,28 @@ void TitlesetManager::AddVideo(const wxString& fname, bool createTitle)
     vob->SetFilename(fname);
   else // video
     vob->SetVideoFilename(fname);
-  vob->SetChapters(
-    _T("15:00,30:00,45:00,1:00:00,1:15:00,1:30:00,1:45:00,2:00:00"));
+  int n = 10;
+  if (vob->GetDuration() > 0) {
+    int duration = (int) vob->GetDuration() / 60;
+  	n = duration / s_config.GetDefChapterLength();
+  	if (n > 0 && duration - s_config.GetDefChapterLength()*n < s_config.GetDefChapterLength()/5)
+  	  n--;
+  }
+  if (n>100)
+    n = 100;
+  int t = s_config.GetDefChapterLength();
+  wxString cStr;
+  for (int i = 0; i < n; i++)
+  {
+    if (cStr.length())
+      cStr += wxT(",");
+    if (t >= 60)
+      cStr += wxString::Format(wxT("%d:%02d:00"), t/60, t%60);
+    else
+      cStr += wxString::Format(wxT("%02d:00"), t%60);
+    t += s_config.GetDefChapterLength();
+  }
+  vob->SetChapters(cStr);
   if (createTitle || pgc == NULL)
   {
     pgc = new Pgc;
@@ -435,9 +490,10 @@ void TitlesetManager::AddVideo(const wxString& fname, bool createTitle)
   
   UpdateItems();
   SendDvdChangedEvent();
+  return true;
 }
 
-void TitlesetManager::AddAudio(const wxString& fname)
+bool TitlesetManager::AddAudio(const wxString& fname)
 {
   if (m_dvd->GetTitlesets().Count() == 0)
     m_dvd->GetTitlesets().Add(new Titleset);
@@ -446,15 +502,16 @@ void TitlesetManager::AddAudio(const wxString& fname)
   if (pgcs.Count()==0 || pgcs.Last()->GetVobs().Count()==0)
   {
     wxLogError(_("Please add a video track first."));
-    return;
+    return false;
   }
   Vob* vob = pgcs.Last()->GetVobs().Last();
   vob->GetAudioFilenames().Add(fname);
   UpdateItems();
   SendDvdChangedEvent();
+  return true;
 }
 
-void TitlesetManager::AddSubtitles(const wxString& fname)
+bool TitlesetManager::AddSubtitles(const wxString& fname)
 {
   if (m_dvd->GetTitlesets().Count() == 0)
     m_dvd->GetTitlesets().Add(new Titleset);
@@ -463,12 +520,13 @@ void TitlesetManager::AddSubtitles(const wxString& fname)
   if (pgcs.Count()==0 || pgcs.Last()->GetVobs().Count()==0)
   {
     wxLogError(_("Please add a video track first."));
-    return;
+    return false;
   }
   Vob* vob = pgcs.Last()->GetVobs().Last();
   vob->GetSubtitles().Add(new TextSub(fname));
   UpdateItems();
   SendDvdChangedEvent();
+  return true;
 }
 
 void TitlesetManager::AddImage(const wxString& fname, bool createTitle)
@@ -672,6 +730,13 @@ void TitlesetManager::OnDelete(wxCommandEvent& WXUNUSED(event))
   int pgci = DVD::GetPgci(id);
   bool isMenu = DVD::IsMenu(id);
   int vobi = DVD::GetVobi(id);
+  wxString caption = isMenu ? _("Menu") : _T("Title");
+  caption += wxString::Format(wxT(" %d"), pgci+1);
+  if (vobi>0)
+  	caption += wxString::Format(_T("-%d"), vobi+1);
+  if (wxMessageBox(wxString::Format(_("Do you really want to delete %s?"), caption.c_str()),
+				_("Titleset Manager"), wxYES_NO|wxICON_QUESTION, this) == wxNO)
+	  return;
   if (isMenu)
   {
 	Titleset* ts = NULL;
@@ -784,3 +849,35 @@ void TitlesetManager::OnProps(wxCommandEvent& WXUNUSED(event))
 	  SendDvdChangedEvent();
   }
 }
+
+void TitlesetManager::OnCopy(wxCommandEvent& WXUNUSED(event)) {
+	int id = GetSelectedId();
+	int tsi = DVD::GetTsi(id);
+	int pgci = DVD::GetPgci(id);
+	int vobi = DVD::GetVobi(id);
+	Vob* vob = m_dvd->GetVob(tsi, DVD::IsMenu(id), pgci, vobi);
+	if (!vob || !vob->GetMenu())
+		return;
+	CopyXmlToClipboard(vob->GetMenu()->GetXML(DVDSTYLER_XML, 0, NULL),
+			DATAFORMAT_MENU);
+}
+
+void TitlesetManager::OnPaste(wxCommandEvent& WXUNUSED(event)) {
+	int tsi = m_dvd->GetTitlesets().Count()-1;
+	if (GetSelected()>=0)
+		tsi = DVD::GetTsi(GetSelectedId());
+	Menu* menu = new Menu(m_dvd->GetVideoFormat());
+	wxSvgXmlDocument doc;
+	if (!GetXmlFromClipboard(DATAFORMAT_MENU, doc))
+		return;
+	menu->PutXML(doc.GetRoot());
+	int pgci = m_dvd->AddMenu(menu, tsi);
+	UpdateItems();
+	SelectMenu(tsi, pgci);
+	SendDvdChangedEvent();
+}
+
+void TitlesetManager::OnUpdateUIObjectPaste(wxUpdateUIEvent& event) {
+	event.Enable(wxTheClipboard->IsSupported(wxDataFormat(DATAFORMAT_MENU)));
+}
+
