@@ -2,8 +2,8 @@
 // Name:        Thumbnails.cpp
 // Purpose:     wxThumbnails class
 // Author:      Alex Thuering
-// Created:	2.02.2003
-// RCS-ID:      $Id: Thumbnails.cpp,v 1.6 2007/02/17 08:57:19 ntalex Exp $
+// Created:		2.02.2003
+// RCS-ID:      $Id: Thumbnails.cpp,v 1.10 2008/02/24 09:50:51 ntalex Exp $
 // Copyright:   (c) Alex Thuering
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -11,15 +11,12 @@
 #include "Thumbnails.h"
 #include "ThumbnailFactory.h"
 #include "ImageProc.h"
+#include "utils.h"
 #include <wx/filename.h>
 #include <wx/dnd.h>
 
-#ifdef __WXMSW__
-#undef  wxBITMAP
-#define wxBITMAP(icon) wxBitmap(icon##_xpm)
-#endif
-#include "rc/noread.xpm"
-#include "rc/gnome-mime-video-mpeg.xpm"
+#include "rc/noread.png.h"
+#include "rc/video.png.h"
 
 /////////////////////////////// wxThumb //////////////////////////////////////
 
@@ -30,14 +27,14 @@ wxBitmap wxThumb::GetBitmap(wxThumbnails* parent, int width, int height)
     wxImage img;
 	if (!m_image.Ok())
 	{
-	  img = wxThumbnailFactory::GetThumbnail(parent, m_filename, width, height);
+	  img = wxThumbnailFactory::GetThumbnail(parent, m_filename, 128, 128);
 	  if (!img.Ok())
 	  {
 		if (!m_filename.length() ||
 			wxImage::FindHandler(m_filename.AfterLast('.').Lower(), -1))
-		  img = wxBITMAP(noread).ConvertToImage();
+		  img = wxBITMAP_FROM_MEMORY(noread).ConvertToImage();
 		else
-		  img = wxBITMAP(gnome_mime_video_mpeg).ConvertToImage();
+		  img = wxBITMAP_FROM_MEMORY(video).ConvertToImage();
 	  }
 	}
 	else
@@ -137,6 +134,7 @@ BEGIN_EVENT_TABLE(wxThumbnails, wxScrolledWindow)
   EVT_MOTION(wxThumbnails::OnMouseMove)
   EVT_LEAVE_WINDOW(wxThumbnails::OnMouseLeave)
   EVT_THUMBNAILS_THUMB_CHANGED(-1, wxThumbnails::OnThumbChanged)
+  EVT_KEY_UP(wxThumbnails::OnKeyUp)
 END_EVENT_TABLE()
 
 DEFINE_EVENT_TYPE(EVT_COMMAND_THUMBNAILS_SEL_CHANGED)
@@ -162,6 +160,7 @@ wxThumbnails::wxThumbnails(wxWindow* parent, int id):
   m_orient = wxTHUMB_VERTICAL;
   ShowFileNames(true);
   SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_LISTBOX));
+  wxThumbnailFactory::Init();
 }
 
 wxThumbnails::~wxThumbnails()
@@ -432,7 +431,6 @@ void wxThumbnails::DrawThumbnail(wxBitmap& bmp, wxThumb& thumb, int index)
 {
   wxMemoryDC dc;
   dc.SelectObject(bmp);
-  dc.BeginDrawing();  
   int x = m_tBorder/2;
   int y = m_tBorder/2;
   // background
@@ -506,8 +504,6 @@ void wxThumbnails::DrawThumbnail(wxBitmap& bmp, wxThumb& thumb, int index)
 	int ty = y + m_tHeight + i*m_tTextHeight + (m_tTextHeight-sh)/2;
 	dc.DrawText(caption, tx, ty);
   }
-  
-  dc.EndDrawing();
   dc.SelectObject(wxNullBitmap);
 }
 
@@ -517,7 +513,6 @@ void wxThumbnails::OnPaint(wxPaintEvent& WXUNUSED(event))
   
   wxPaintDC dc(this);  
   PrepareDC(dc);
-  dc.BeginDrawing();
   // items
   int row = -1;
   for (int i=0; i<(int)m_items.GetCount(); i++)
@@ -556,7 +551,6 @@ void wxThumbnails::OnPaint(wxPaintEvent& WXUNUSED(event))
     rect.y += (m_rows-1)*(m_tHeight+m_tBorder)+GetCaptionHeight(0,m_rows-1);
     dc.DrawRectangle(rect);
   }
-  dc.EndDrawing();
 }
 
 void wxThumbnails::OnResize(wxSizeEvent &event)
@@ -736,3 +730,36 @@ void wxThumbnails::OnThumbChanged(wxCommandEvent &event)
 	}
   Refresh();
 }
+
+void wxThumbnails::OnKeyUp(wxKeyEvent& event) {
+	int oldSelected = m_selected;
+	switch (event.GetKeyCode()) {
+		case WXK_LEFT:
+			if (m_selected > 0)
+				m_selected--;
+			break;
+		case WXK_RIGHT:
+			if (m_selected < (int)m_items.GetCount()-1)
+				m_selected++;
+			break;
+		case WXK_UP:
+			if (m_selected >= m_cols)
+				m_selected -= m_cols;
+			break;
+		case WXK_DOWN:
+			if (m_selected + m_cols <= (int)m_items.GetCount()-1)
+				m_selected += m_cols;
+			break;
+		default:
+			event.Skip();
+			break;
+	}
+	if (m_selected != oldSelected) {
+		m_selectedArray.Clear();
+		m_selectedArray.Add(m_selected);
+		ScrollToSelected();
+		Refresh();
+		wxCommandEvent evt(EVT_COMMAND_THUMBNAILS_SEL_CHANGED, this->GetId());
+		GetEventHandler()->ProcessEvent(evt);
+	}
+}
